@@ -17,11 +17,14 @@ import math
 import psutil
 import platform
 import glob
+import ctypes
 from pathlib import Path
-from uuid import getnode as get_mac
 from traceback import TracebackException
 # noinspection PyProtectedMember
-from pip import _internal
+# from pip import _internal
+import subprocess
+import pkg_resources
+from enum import Enum
 
 LINES = '-' * 80
 NOT_FOUND = '{} Not found'
@@ -105,7 +108,7 @@ CONST_COLOR_MAP = {
 
 def get_linkable_exception() -> str:
     """
-    :return: e.g. File "D:\workspace\2021wizzi_utils\wizzi_utils\misc\test\test_misc_tools.py", line 851,
+    :return: e.g. File "D:/workspace/2021wizzi_utils/wizzi_utils/misc/test/test_misc_tools.py", line 851,
                             in get_linkable_exception_test: division by zero
     where file is clickable
     see get_linkable_exception_test
@@ -235,24 +238,6 @@ def get_pc_name(ack: bool = False, tabs: int = 1) -> str:
     return pc_name
 
 
-def get_mac_address(with_colons: bool = True, ack: bool = False, tabs: int = 1) -> str:
-    """
-    :return: pc mac as str
-    see get_mac_address_test()
-    """
-    try:
-        mac = get_mac()
-        mac = "{:X}".format(mac)  # turn to Hex
-        if with_colons:
-            mac = ':'.join(mac[i:i + 2] for i in range(0, 12, 2))
-        if ack:
-            print('{}* Computer Mac: {}'.format(tabs * '\t', mac))
-    except ModuleNotFoundError as e:
-        mac = ''
-        exception_error(e, real_exception=True)
-    return mac
-
-
 def get_cudnn_version(cudnn_path: str) -> str:
     cudnn_v = None
     if os.path.exists(cudnn_path):
@@ -397,7 +382,7 @@ def del_env_variable(key: str, ack: bool = False, tabs: int = 1) -> None:
 def silence_cv_warn() -> None:
     """
     disables on windows:
-    [ WARN:0] global .... src\cap_msmf.cpp (438)
+    [ WARN:0] global .... src/cap_msmf.cpp (438)
     `anonymous-namespace'::SourceReaderCB::~SourceReaderCB terminating async callback
     :return:
     if doesn't work on linux try:
@@ -469,7 +454,7 @@ def version() -> str:
     :return:
     """
     # v = pkg_resources.require("wizzi_utils")[0].version
-    v = '7.0.10'
+    v = '7.0.17'
     return v
 
 
@@ -610,15 +595,20 @@ def main_wrapper(
     print('* Interpreter: {}'.format(sys.executable))
     print('* wizzi_utils Version {}'.format(version()))
     print('* Working Dir: {}'.format(os.getcwd()))
-    print('* Computer Mac: {}'.format(get_mac_address()))
+    try:
+        from wizzi_utils.socket.socket_tools import get_mac_address_uuid
+        print('* Computer Mac: {}'.format(get_mac_address_uuid()))
+    except (ImportError, ModuleNotFoundError, NameError, AttributeError) as err:
+        print(add_color('* {}'.format(err), 'r'))
+   
     print('* CPU Info: {}'.format(cpu_info(one_liner=True, tabs=0)))
     print('* Physical Memory: {}'.format(hard_disc(one_liner=True, tabs=0)))
     print('* RAM: {}'.format(ram_size(one_liner=True, tabs=0)))
 
     if ipv4:
         try:
-            from wizzi_utils.socket.socket_tools import get_ipv4
-            print('* Computer ipv4: {}'.format(get_ipv4()))
+            from wizzi_utils.socket.socket_tools import get_active_ipv4
+            print('* Computer ipv4: {}'.format(get_active_ipv4()))
         except (ImportError, ModuleNotFoundError, NameError, AttributeError) as err:
             print(add_color('* {}'.format(err), 'r'))
 
@@ -661,7 +651,10 @@ def main_wrapper(
 
     if with_pip_list:
         print('* Pip list output:')
-        _internal.main(['list'])
+        # _internal.main(['list'])
+        pkgs = get_pip_freeze_out(ack=False)
+        for i, pkg in enumerate(pkgs):
+            print('\t{}'.format(pkg))
 
     print('Function {} started:'.format(main_function))
     print(LINES)
@@ -739,7 +732,7 @@ def to_str(var: any,
     :param fp: float_precision: round number if possible(float, list or np array of floats...)
             fp>=0 round
             fp==-1: no rounding
-    :param wm: with_meta: with meta data such as type, len\shape, dtype...
+    :param wm: with_meta: with meta data such as type, len/shape, dtype...
     :param rec: recursive: to keep printing if there are more items inside e.g. np.array(shape=(2,3,4)) -> 3 prints
     :return: informative string of the variable
     see to_str_test()
@@ -1234,7 +1227,7 @@ def get_base_file_and_function_name(depth: int = 1, ack: bool = False, tabs: int
 
 def add_color(
         string: str,
-        ops: (str, list) = 'red'
+        ops: (str, list) = None
 ) -> str:
     """
     :param string:
@@ -1257,12 +1250,15 @@ def add_color(
             print('{}{}{}'.format(v, k, wu.CONST_COLOR_MAP['reset_all']))
     see add_color_test()
     """
-    out_string = string
-    # first to lower and check shortcuts
-    if is_list(ops):
-        pass
+    if ops is None:
+        ops = ['r']
     elif is_str(ops):
         ops = [ops]
+    elif is_list(ops):
+        pass  # good to go
+
+    out_string = string
+    # first to lower and check shortcuts
     colors = ''
     for op in ops:
         op = op.lower()
@@ -1375,7 +1371,7 @@ def delete_empty_dir(dir_path: str, ack: bool = True, tabs: int = 1):
                 # fp = '{}/{}'.format(_dir_path.replace('\\', '/'), f)
                 files_and_dirs.append(f)
         if len(files_and_dirs) > 0:
-            err = '{} HAS {} FILES\DIRS {} - use delete_dir_with_files()'.format(dir_path, len(files_and_dirs),
+            err = '{} HAS {} FILES/DIRS {} - use delete_dir_with_files()'.format(dir_path, len(files_and_dirs),
                                                                                  files_and_dirs)
             exception_error(err, real_exception=False, tabs=tabs)
         else:
@@ -1410,7 +1406,7 @@ def delete_dir_with_files(dir_path: str, ack: bool = True, tabs: int = 1):
         try:
             size_s = file_or_folder_size(dir_path)
             shutil.rmtree(dir_path)
-            status = '({} files\dirs, size {} - {})'.format(len(files_and_dirs), size_s, files_and_dirs)
+            status = '({} files/dirs, size {} - {})'.format(len(files_and_dirs), size_s, files_and_dirs)
             if ack:
                 print('{}{} - {}'.format(tabs * '\t', DELETED.format(dir_path), status))
         except OSError as e:
@@ -1909,8 +1905,10 @@ def cpu_info(one_liner: bool = False, tabs: int = 1):
         string += '\n{}\tMin Frequency {:.2f}Mhz'.format(tabs * '\t', cpufreq.min)
         string += '\n{}\tCurrent Frequency {:.2f}Mhz'.format(tabs * '\t', cpufreq.current)
         string += '\n{}\tCPU Usage {}%'.format(tabs * '\t', psutil.cpu_percent())
+        # noinspection PyTypeChecker
+        per_core_raw: list = psutil.cpu_percent(percpu=True, interval=1)
         per_core = ''
-        for i, percentage in enumerate(psutil.cpu_percent(percpu=True, interval=1)):
+        for i, percentage in enumerate(per_core_raw):
             # print(f"Core {i}: {percentage}%")
             per_core += 'C{}-{}% '.format(i, percentage)
         string += '\n{}\tCPU Usage per core {}'.format(tabs * '\t', per_core)
@@ -1918,127 +1916,13 @@ def cpu_info(one_liner: bool = False, tabs: int = 1):
 
 
 def wizzi_utils_requirements():
-    pip = 'pip install {}'
-    pack = 'Package {}({}.py, namespace: {}):'
-    pack_test = '\ttest_{}.py(namespace: {}.test):'
-    sub_pack = 'Sub Package: {}'
-
-    MT_PACK, MT_MOD, MT_SC = 'misc', 'misc_tools', 'mt'
-    # ALG_PACK, ALG_MOD, ALG_SC = 'algorithms', 'algorithms', 'alg'
-    # COT_PACK, COT_MOD, COT_SC = 'coreset', 'coreset_tools', 'cot'
-    GOT_PACK, GOT_MOD, GOT_SC = 'google', 'google_tools', 'got'
-    JT_PACK, JT_MOD, JT_SC = 'json', 'json_tools', 'jt'
-    CVT_PACK, CVT_MOD, CVT_SC = 'open_cv', 'open_cv_tools', 'cvt'
-    PYPLT_PACK, PYPLT_MOD, PYPLT_SC = 'pyplot', 'pyplot_tools', 'pyplt'
-    ST_PACK, ST_MOD, ST_SC = 'socket', 'socket_tools', 'st'
-    TT_PACK, TT_MOD, TT_SC = 'torch', 'torch_tools', 'tt'
-    # TFT_PACK, TFT_MOD, TFT_SC = 'tensorflow', 'tensorflow_tools', 'tft'
-    TFLT_PACK, TFLT_MOD, TFLT_SC = 'tflite', 'tflite_tools', 'tflt'
-    TTS_PACK, TTS_MOD, TTS_SC = 'tts', 'tts', 'tts'
-    MODELS_PACK, MODELS_MOD, MODELS_SC = 'models', 'models', 'models'
-    print('Requirements per package:')
-    print('\tTo use a test of a package, must install all package requirements')
-
-    print(LINES)
-    print(pack.format(MT_PACK, MT_MOD, MT_SC))
-    print('\t{} # to use 7z compress and extract'.format(pip.format('py7zr')))
-    print(pack_test.format(MT_MOD, MT_SC))
-    print('\t\tNone')
-
-    # print(LINES)
-    # print(pack.format(ALG_PACK, ALG_MOD, ALG_SC))
-    # print('\t{}'.format(pip.format('sklrean')))
-    # print(pack_test.format(ALG_MOD, ALG_SC))
-    # print('\t\t{}'.format(sub_pack.format(PYPLT_PACK)))
-
-    # print(LINES)
-    # print(pack.format(COT_PACK, COT_MOD, COT_SC))
-    # print('\tNone')
-    # print(pack_test.format(COT_MOD, COT_SC))
-    # print('\t\tNone')
-
-    print(LINES)
-    print(pack.format(GOT_PACK, GOT_MOD, GOT_SC))
-    print('\t*** see instructions in order to use written in class google_handler docstring')
-    print('\t{}'.format(pip.format('pydrive')))
-    print('\t{}'.format(pip.format('yaml')))
-    print(pack_test.format(GOT_MOD, GOT_SC))
-    print('\t\t{}'.format(sub_pack.format(ST_PACK)))
-
-    print(LINES)
-    print(pack.format(JT_PACK, JT_MOD, JT_SC))
-    print('\tNone')
-    print(pack_test.format(JT_MOD, JT_SC))
-    print('\t\tNone')
-
-    print(LINES)
-    print(pack.format(CVT_PACK, CVT_MOD, CVT_SC))
-    print('\t{}'.format(pip.format('opencv-python')))
-    print('\t{}'.format(sub_pack.format(PYPLT_PACK)))
-    print('\t{} # for CameraWu type_cam=="imutils"'.format(pip.format('imutils')))
-    print('\t{} # for CameraWu type_cam=="acapture"'.format(pip.format('acapture')))
-    print(pack_test.format(CVT_MOD, CVT_SC))
-    print('\t\t{}'.format(sub_pack.format(ST_PACK)))
-
-    print(LINES)
-    print(pack.format(PYPLT_PACK, PYPLT_MOD, PYPLT_SC))
-    print('\t{} # for compare_images_multi_sets_squeezed()'.format(pip.format('torch')))
-    print(pack_test.format(PYPLT_MOD, PYPLT_SC))
-    print('\t\t{} # for compare_images_multi_sets_squeezed_test()'.format(pip.format('torch')))
-    print('\t\t{} # with the tests as well'.format(sub_pack.format(CVT_PACK)))
-
-    print(LINES)
-    print(pack.format(ST_PACK, ST_MOD, ST_SC))
-    print('\tNone')
-    print(pack_test.format(ST_MOD, ST_SC))
-    print('\t\t{}'.format(sub_pack.format(JT_PACK)))
-
-    print(LINES)
-
-    print(pack.format(TT_PACK, TT_MOD, TT_SC))
-    print('\ttorch:')
-    print('\t\t{} # cpu'.format(pip.format('torch')))
-    print('\t\tfor gpu support check your cuda and google for the pip command')
-    print('\t\tall options: https://pytorch.org/get-started/locally/')
-    print('\t\t** may need to uninstall numpy and re-install')
-    print('\t{}'.format(pip.format('torchvision')))
-    print(pack_test.format(TT_MOD, TT_SC))
-    print('\t\tNone')
-
-    # print(LINES)
-    # print(pack.format(TFT_PACK, TFT_MOD, TFT_SC))
-    # print('\ttensorflow:')
-    # print('\t\t{}'.format(pip.format('tensorflow')))
-    # url = 'https://stackoverflow.com/questions/50622525/'
-    # url += 'which-tensorflow-and-cuda-version-combinations-are-compatible'
-    # print('\t\tall options: {}'.format(url))
-    # print(pack_test.format(TFT_MOD, TFT_SC))
-    # print('\t\tNone')
-
-    print(LINES)
-    print(pack.format(TFLT_PACK, TFLT_MOD, TFLT_SC))
-    print('\ttflite options:')
-    url_win10 = 'https://google-coral.github.io/py-repo/'
-    print('\t\t{} # windows 10 and linux'.format(pip.format('--index-url {} tflite_runtime'.format(url_win10))))
-    print('\t\t\tmore info on https://www.tensorflow.org/lite/guide/python')
-
-    print(pack_test.format(TFLT_MOD, TFLT_SC))
-    print('\t\t{}'.format(None))
-
-    print(LINES)
-    print(pack.format(TTS_PACK, TTS_MOD, TTS_SC))
-    print('\t{}'.format(pip.format('pyttsx3')))
-    print(pack_test.format(TTS_MOD, TTS_SC))
-    print('\t\tNone')
-
-    print(LINES)
-    print(pack.format(MODELS_PACK, MODELS_MOD, MODELS_SC))
-    print('\t{}'.format(sub_pack.format(ST_PACK)))
-    print('\t{}'.format(sub_pack.format(PYPLT_PACK)))
-    print('\t{}'.format(sub_pack.format(CVT_PACK)))
-    print('\t{} # for tracking'.format(pip.format('opencv-contrib-python')))
-    print('\tinstall tflite for od and pd models')
-    print(LINES)
+    print('A snapshot of my environment packages:')
+    req_file = '../wizzi_utils_requirements.txt'
+    file = open(req_file, 'r')
+    lines = file.readlines()
+    file.close()
+    for i, line in enumerate(lines):
+        print("{}) {}".format(i + 1, line.rstrip()))
     return
 
 
@@ -2192,11 +2076,16 @@ class FPS:
     see classFPS_test()
     """
 
-    def __init__(self, summary_title: str = None) -> None:
+    def __init__(self, last_k: int = 100, summary_title: str = None) -> None:
+        """
+        :param last_k: average of the last k measures
+        :param summary_title:
+        """
         self.timer_begin = None
         self.timer_end = None
         self.times_elapsed = []
         self.summary_title = summary_title
+        self.last_k = last_k
         return
 
     def __del__(self) -> None:
@@ -2205,10 +2094,6 @@ class FPS:
     def __str__(self) -> str:
         return ''
 
-    def _times_elapsed_not_empty(self) -> bool:
-        ret = True if len(self.times_elapsed) > 0 else False
-        return ret
-
     def start(self, ack_progress: bool = False, tabs: int = 1, with_title: bool = False) -> None:
         if ack_progress:
             string = tabs * '\t'
@@ -2227,6 +2112,7 @@ class FPS:
             self.timer_end = get_timer()
             time_elapsed = self.timer_end - self.timer_begin
             self.times_elapsed.append(time_elapsed)
+
             if ack_progress:
                 self.so_far(tabs, with_title)
             self.timer_begin = None
@@ -2235,14 +2121,14 @@ class FPS:
 
     def get_fps(self) -> float:
         fps = -1.0
-        if self._times_elapsed_not_empty():
-            avg_times = np.mean(self.times_elapsed)
+        if len(self.times_elapsed) > 0:
+            avg_times = np.mean(self.times_elapsed[-self.last_k:])
             if avg_times > 0.0001:
                 fps = 1 / avg_times
         return fps
 
     def so_far(self, tabs: int = 1, with_title: bool = False) -> None:
-        if self._times_elapsed_not_empty():
+        if len(self.times_elapsed) > 0:
             string = tabs * '\t'
             if with_title:
                 string += '{}:'.format(self.summary_title)
@@ -2252,9 +2138,9 @@ class FPS:
             rt_str = '{:.3f}s'.format(self.times_elapsed[-1])
 
             string += 'iterTime={}, '.format(rt_str)
-            avg_times = np.mean(self.times_elapsed)
+            avg_times = np.mean(self.times_elapsed[-self.last_k:])
             if len(self.times_elapsed) > 1:
-                string += 'totalTime={:.3f}s, '.format(np.sum(self.times_elapsed))
+                string += 'totalTime={:.3f}s, '.format(np.sum(self.times_elapsed[-self.last_k:]))
                 string += 'avgTime={:.3f}s, '.format(avg_times)
 
             if avg_times > 0.0001:
@@ -2271,7 +2157,7 @@ class FPS:
 
     def finalize(self, tabs: int = 1) -> None:
         tabs_s = tabs * '\t'
-        if self._times_elapsed_not_empty():
+        if len(self.times_elapsed) > 0:
             # title
             title = 'Summary '
             if self.summary_title is not None:
@@ -2287,14 +2173,336 @@ class FPS:
             # average time
             avg_times = np.mean(self.times_elapsed)
             std_times = np.std(self.times_elapsed)
-            avg_str = '{}\tAverage Run Time = {:.3f}s (std = {:.3f})'.format(tabs_s, avg_times, std_times)
-            print(avg_str)
+            avg_str = '{}\tAverage Run Time(all t) = {:.3f}s (std = {:.3f})'.format(tabs_s, avg_times, std_times)
+
             # FPS
             if avg_times > 0.0001:
-                fps = '{}\t{:.2f} FPS'.format(tabs * '\t', 1 / avg_times)
+                fps = ' {:.2f} FPS'.format(1 / avg_times)
             else:
-                fps = '{}\tFPS very high'.format(tabs * '\t')
-            print(add_color(fps, ops='r'))
+                fps = 'FPS very high'
+            avg_str += add_color(fps, ops='r')
+
+            print(avg_str)
+
+            # average time last k
+            avg_times_k = np.mean(self.times_elapsed[-self.last_k:])
+            std_times_k = np.std(self.times_elapsed[-self.last_k:])
+            avg_str_k = '{}\tAverage Run Time(last {}) = {:.3f}s (std = {:.3f})'.format(tabs_s, self.last_k,
+                                                                                        avg_times_k,
+                                                                                        std_times_k)
+            # FPS last k
+            if avg_times > 0.0001:
+                fps_k = ' {:.2f} FPS'.format(1 / avg_times_k)
+            else:
+                fps_k = 'FPS very high'
+            avg_str_k += add_color(fps_k, ops='r')
+
+            if fps_k != fps:
+                print(avg_str_k)
+
         else:
             exception_error('Times_elapsed is empty', real_exception=False, tabs=tabs)
         return
+
+
+def run_shell_command(cmd: str, ack: bool = True) -> None:
+    try:
+        if ack:
+            print(add_color(cmd, ops='b'))
+        subprocess.check_call(cmd, shell=True)
+
+    except subprocess.CalledProcessError as e:
+        exception_error(e, real_exception=True)
+    return
+
+
+def run_shell_command_and_get_out(cmd: str, ack_cmd: bool = True, ack_out: bool = False) -> list:
+    out = ['Failed']
+    try:
+        if ack_cmd:
+            print(add_color(cmd, ops='b'))
+        out = subprocess.check_output(cmd, shell=True)
+        out = out.decode("utf-8").replace('\t', '').replace('\r', '').strip()
+        out = out.split('\n')
+        if ack_out:
+            for i, line in enumerate(out):
+                print('{}){}'.format(i, line))
+    except subprocess.CalledProcessError as e:
+        exception_error(e, real_exception=True)
+    return out
+
+
+def add_sudo_to_cmd(cmd: str, sudo_password: str) -> str:
+    cmd_wrap = 'echo {}|sudo -S {}'.format(sudo_password, cmd)
+    return cmd_wrap
+
+
+def get_pip_freeze_out(ack: bool = True) -> list:
+    """
+    you can check out
+    https://stackoverflow.com/questions/31684375/automatically-create-requirements-txt
+    many solutions on pip freeze
+    :param ack:
+    :return:
+    """
+    installed_packages = pkg_resources.working_set
+    installed_packages_list = sorted(["%s==%s" % (i.key, i.version) for i in installed_packages])
+    if ack:
+        print('pip freeze out:')
+        for i, pkg in enumerate(installed_packages_list):
+            print('{}){}'.format(i, pkg))
+    return installed_packages_list
+
+
+def remove_colors(string: str) -> str:
+    # print(string)
+    # print(repr(string))
+    for name, value in CONST_COLOR_MAP.items():
+        # print(name)
+        # print(value)
+        if value in string:
+            string = string.replace(value, '')
+    # print(string)
+    # print(repr(string))
+    return string
+
+
+def generate_requirements_file(fp_out: str, ack: bool = True) -> None:
+    """
+    :param fp_out: full path of output file.
+    :param ack:
+    :return:
+    """
+    d_name = os.path.dirname(fp_out)
+    if not os.path.exists(d_name):
+        exception_error(e='folder doesn\'t exists: {}'.format(os.path.abspath(d_name)))
+    else:
+        out = open(file=fp_out, mode='w', encoding='utf-8')
+        # _internal.main(['list'])
+        # _internal.main(['freeze'])
+        # run_shell_command(cmd='pip list', ack=True)
+        # out = run_shell_command_and_get_out(cmd='pip list', ack_cmd=True, ack_out=True)
+        # run_shell_command(cmd="which python", ack=True)
+
+        # init_logger()
+        # main_wrapper()
+        out.write('# main_wrapper:\n')
+        out.write('# * Run started at {}\n'.format(get_time_stamp(format_s='%d-%m-%Y %H:%M:%S')))
+        py_ver = sys.version.replace('\n', '')  # on linux there is \n and then gcc
+        out.write('# * Python Version {}\n'.format(py_ver))
+        out.write('# * Operating System {}\n'.format(remove_colors(get_system_info())))
+        out.write('# * Interpreter: {}\n'.format(sys.executable))
+        out.write('# * wizzi_utils Version {}\n'.format(version()))
+        out.write('# * Working Dir: {}\n'.format(os.getcwd()))
+        out.write('# * Computer Mac: {}\n'.format(remove_colors(get_mac_address())))
+        out.write('# * CPU Info: {}\n'.format(remove_colors(cpu_info(one_liner=True, tabs=0))))
+        out.write('# * Physical Memory: {}\n'.format(remove_colors(hard_disc(one_liner=True, tabs=0))))
+        out.write('# * RAM: {}\n'.format(remove_colors(ram_size(one_liner=True, tabs=0))))
+
+        try:
+            from wizzi_utils.socket.socket_tools import get_active_ipv4
+            out.write('# * Computer ipv4: {}\n'.format(get_active_ipv4()))
+        except (ImportError, ModuleNotFoundError, NameError, AttributeError) as err:
+            out.write('# * {}\n'.format(err))
+
+        cuda_msg = get_cuda_version(ack=False, tabs=0)
+        cuda_msg = remove_colors(cuda_msg)
+        out.write('# {}\n'.format(cuda_msg))
+
+        try:
+            from wizzi_utils.open_cv.open_cv_tools import get_cv_version
+            cv_v = get_cv_version(ack=False, tabs=0)
+            cv_v = remove_colors(cv_v)
+            out.write('# {}\n'.format(cv_v))
+        except (ImportError, ModuleNotFoundError, NameError, AttributeError) as err:
+            out.write('# * opencv-python not Found. error: {}\n'.format(err))
+
+        pkgs = get_pip_freeze_out(ack=False)
+        # TODO future: remove wheel, pip, setuptools ???
+        # they are missing on pip freeze and appear here
+        for pkg in pkgs:
+            out.write('{}\n'.format(pkg))
+
+        out.close()
+        if ack:
+            fp_out = os.path.abspath(fp_out)
+            file_or_folder_size(path=fp_out, ack=True)
+    return
+
+
+def size_remain(partition: str, ack: bool = False) -> (int, str):
+    """
+    :param partition:
+    e.g. windows: partition='C:' partition='D:'
+         linux:   partition='/dev/root'
+    :param ack:
+    :return:
+    """
+    partitions = psutil.disk_partitions()
+    free_space_int = None
+    free_space_str = None
+    for i, current_partition in enumerate(partitions):
+        # print(partition, current_partition)
+        if partition in current_partition.device:
+            try:
+                # print(current_partition.device)
+                partition_usage = psutil.disk_usage(current_partition.mountpoint)
+                free_space_int = partition_usage.free
+                free_space_str = convert_size(free_space_int)
+                if ack:
+                    print('\ton {} free space int={}, str={}'.format(partition, free_space_int, free_space_str))
+            except PermissionError as e:
+                exception_error('PermissionError: {}'.format(e))
+
+    return free_space_int, free_space_str
+
+
+def size_remain_greater_than(partition: str, gb: int, mb: int, ack: bool = False) -> bool:
+    """
+    check if partition has more than `gb` + `mb` free
+    :param partition:
+    e.g. windows: partition='C:' partition='D:'
+         linux:   partition='/dev/root'
+    :param gb:
+    :param mb:
+    :param ack:
+    :return:
+    """
+    greater_than = False
+    # first get remaining space on partition
+    free_space_int, free_space_str = size_remain(partition=partition, ack=False)
+    if free_space_int is not None:
+        if ack:
+            # convert gb and mb into bytes
+            size_bites_int = convert_size_to_bytes(size_gb=gb, size_mb=mb)
+            if size_bites_int > 0:
+                combined_size = gb + mb / 1000
+                info = 'on {} free space {}'.format(partition, free_space_str)
+                if free_space_int > size_bites_int:
+                    info += ' GREATER than'
+                    greater_than = True
+                else:
+                    info += ' SMALLER than'
+
+                info += ' {} GB'.format(combined_size)
+                if greater_than:
+                    info = add_color(info, ops=SUCCESS_C2)
+                else:
+                    info = add_color(info, ops=FAIL_C2)
+                print('\t{}'.format(info))
+    return greater_than
+
+
+def convert_size_to_bytes(size_gb: int, size_mb: int) -> int:
+    """
+    :param size_gb:  size in gigabytes
+    :param size_mb:  size in gigabytes
+    :return:
+    size_gb = 1.0 -> size_bytes = 1073741824
+    """
+    size_bytes: int = 0
+    if size_gb > 0:
+        size_bytes = size_gb * 1024 ** 3
+    if size_mb > 0:
+        size_bytes += size_mb * 1024 ** 2
+    return int(size_bytes)
+
+
+class EMOJIS(Enum):
+    # https://www.geeksforgeeks.org/python-program-to-print-emojis/
+    GRINNING_FACE = "\U0001F600"
+    GRINNING_FACE_WITH_BIG_EYS = "\U0001F603"
+    GRINNING_FACE_WITH_SMILING_EYS = "\U0001F604"
+    GRINNING_SQUINTING_FACE = "\U0001F606"
+    GRINNING_FACE_WITH_SWEAT = "\U0001F605"
+    ROLLING_ON_THE_FLOOR_LAUGHING = "\U0001F923"
+    FACE_WITH_TEARS_OF_JOY = "\U0001F602"
+    SLIGHTLY_SMILING_FACE = "\U0001F642"
+    UPSIDE_DOWN_FACE = "\U0001F643"
+    WINKING_FACE = "\U0001F609"
+    SMILING_FACE_WITH_SMILING_EYES = "\U0001F60A"
+    SMILING_FACE_WITH_HALO = "\U0001F607"
+    SMILING_FACE_WITH_3_HEARTS = "\U0001F970"
+    SMILING_FACE_WITH_HEART_EYES = "\U0001F60D"
+    STAR_STRUCK = "\U0001F929"
+    FACE_BLOWING_A_KISS = "\U0001F618"
+    KISSING_FACE = "\U0001F617"
+    KISSING_FACE_WITH_CLOSED_EYES = "\U0001F61A"
+    KISSING_FACE_WITH_SMILING_EYES = "\U0001F619"
+    FACE_SAVORING_FOOD = "\U0001F60B"
+    FACE_WITH_TONGUE = "\U0001F61B"
+    WINKING_FACE_WITH_TONGUE = "\U0001F61C"
+    ZANY_FACE = "\U0001F92A"
+    SQUINTING_FACE_WITH_TONGUE = "\U0001F61D"
+    MONEY_MOUTH_FACE = "\U0001F911"
+    HUGGING_FACE = "\U0001F917"
+    FACE_WITH_HAND_OVER_MOUTH = "\U0001F92D"
+    SHUSHING_FACE = "\U0001F92B"
+    THINKING_FACE = "\U0001F914"
+    ZIPPER_MOUTH_FACE = "\U0001F910"
+    FACE_WITH_RAISED_EYBROW = "\U0001F928"
+    NATURAL_FACE = "\U0001F610"
+    EXPRESSIONLESS_FACE = "\U0001F611"
+    FACE_WITHOUT_MOUTH = "\U0001F636"
+    SMIRKING_FACE = "\U0001F60F"
+    UNAMUSED_FACE = "\U0001F612"
+    FACE_WITH_ROLLING_EYES = "\U0001F644"
+    GRIMACING_FACE = "\U0001F62C"
+    LYING_FACE = "\U0001F925"
+    RELIEVED_FACE = "\U0001F60C"
+    PENSIVE_FACE = "\U0001F614"
+    SLEEPY_FACE = "\U0001F62A"
+    DROOLING_FACE = "\U0001F924"
+    SLEEPING_FACE = "\U0001F634"
+    FACE_WITH_MEDICAL_MASK = "\U0001F637"
+    FACE_WITH_THERMOMETER = "\U0001F912"
+    FACE_WITH_HEAD_BANDAGE = "\U0001F915"
+    NAUSEATED_FACE = "\U0001F922"
+
+    @staticmethod
+    def get_names():
+        return [em.name for em in EMOJIS]
+
+    @staticmethod
+    def get_values():
+        return [em.value for em in EMOJIS]
+
+    @staticmethod
+    def get_emojis():
+        return [em for em in EMOJIS]
+
+
+def get_emoji(emoji: EMOJIS) -> str:
+    return emoji.value
+
+
+def has_admin(ack: bool = True) -> (str, bool):
+    """
+    checks if user has admin privileges
+    pycharm users:
+        close all pycharm if open
+        right click pycharm icon > run as administrator
+        has_admin should yield True
+    :param ack:
+    :return:
+    TODO test on linux
+    """
+    user_name, is_admin = 'Unknown', False
+    if is_windows():
+        user_name = get_env_variable(key='USERNAME')
+        try:
+            is_admin = ctypes.windll.shell32.IsUserAnAdmin() != 0
+        except Exception as e:
+            exception_error(e, real_exception=True)
+            is_admin = False
+
+    elif is_linux():
+        if 'SUDO_USER' in os.environ and os.geteuid() == 0:
+            user_name = get_env_variable(key='SUDO_USER')
+            is_admin = True
+        else:
+            user_name = get_env_variable(key='USERNAME')
+            is_admin = False
+    if ack:
+        print('Username {} is admin ? {}'.format(user_name, is_admin))
+    return user_name, is_admin
