@@ -114,14 +114,6 @@ def get_pc_name_test():
     return
 
 
-def get_mac_address_test():
-    mt.get_function_name(ack=True, tabs=0)
-    mt.get_mac_address(ack=True, tabs=1)
-    mt.get_mac_address(with_colons=False, ack=True, tabs=1)
-    print('\tMac address is {}'.format(mt.get_mac_address()))
-    return
-
-
 def get_cuda_version_test():
     mt.get_function_name(ack=True, tabs=0)
     mt.get_cuda_version(ack=True, tabs=1)
@@ -224,6 +216,7 @@ def main_wrapper_test():
         torch_v=True,
         tf_v=True,
         cv2_v=True,
+        with_pip_list=True,
         with_profiler=False
     )
     return
@@ -553,9 +546,9 @@ def logger_test():
 
     print('\treading from {}'.format(path))
     file1 = open(path, 'r')
-    Lines = file1.readlines()
+    lines = file1.readlines()
     file1.close()
-    for i, line in enumerate(Lines):
+    for i, line in enumerate(lines):
         print("\t\tLine{}: {}".format(i + 1, line.rstrip()))
     mt.delete_dir_with_files(TEMP_FOLDER1)
     return
@@ -844,17 +837,17 @@ def compress_and_extract_test():
 
 def classFPS_test():
     mt.get_function_name_and_line(ack=True, tabs=0)
-    fps = mt.FPS(summary_title='classFPS_test')
+    fps = mt.FPS(last_k=2, summary_title='classFPS_test')
     tabs = 1
     print('{}test empty get_fps() = {:.4f}'.format(tabs * '\t', fps.get_fps()))
     for t in range(3):
         fps.start(ack_progress=True, tabs=tabs)
         # do_work of round t
         mt.sleep(seconds=0.03)
+        if t == 0:
+            mt.sleep(seconds=1)
         fps.update(ack_progress=True, tabs=tabs + 1)
         print('{}\tget_fps() = {:.4f}'.format(tabs * '\t', fps.get_fps()))
-        if t == 0:
-            fps.clear()
     fps.finalize(tabs)
     return
 
@@ -895,6 +888,118 @@ def get_linkable_exception_test():
     return
 
 
+def generate_requirements_file_test():
+    mt.get_function_name_and_line(ack=True, tabs=0)
+    fp = '{}/requirements.txt'.format(PLAY_GROUND)
+    mt.generate_requirements_file(fp_out=fp)
+    file1 = open(fp, 'r')
+    lines = file1.readlines()
+    file1.close()
+    print('\nfile content:')
+    for i, line in enumerate(lines):
+        print("\t{}".format(line.rstrip()))
+    # mt.delete_file(fp)
+    return
+
+
+def run_shell_command_test():
+    mt.get_function_name_and_line(ack=True, tabs=0)
+    if mt.is_windows():
+        mt.run_shell_command(cmd='dir')
+    else:
+        mt.run_shell_command(cmd='ls')
+    return
+
+
+def run_shell_command_and_get_out_test():
+    mt.get_function_name_and_line(ack=True, tabs=0)
+    if mt.is_windows():
+        out = mt.run_shell_command_and_get_out(cmd='dir', ack_out=False)
+    else:
+        out = mt.run_shell_command_and_get_out(cmd='ls', ack_out=False)
+    for i, line in enumerate(out):
+        print('\t{}){}'.format(i, line))
+    return
+
+
+def add_sudo_to_cmd_test():
+    mt.get_function_name_and_line(ack=True, tabs=0)
+    if mt.is_linux():
+        mt.get_function_name_and_line(ack=True, tabs=0)
+        cmd = 'ls'
+        cmd_sudo = mt.add_sudo_to_cmd(cmd, sudo_password='my_password')
+        mt.run_shell_command(cmd=cmd_sudo)
+    return
+
+
+def size_remain_test():
+    mt.get_function_name_and_line(ack=True, tabs=0)
+    if mt.is_windows():
+        mt.size_remain(partition='C:', ack=True)
+        mt.size_remain(partition='E:', ack=True)
+    else:
+        mt.size_remain(partition='/dev/root', ack=True)
+    return
+
+
+def convert_size_test():
+    mt.get_function_name_and_line(ack=True, tabs=0)
+    gb_bytes = 1073741824  # 1 GB in bytes
+    mb_bytes = 1048576  # 1 MB in bytes
+
+    size_int = gb_bytes
+    size_str = mt.convert_size(size_bytes=size_int)
+    print('\t{} is {}'.format(size_int, size_str))
+    size_int2 = mt.convert_size_to_bytes(size_gb=1, size_mb=0)
+    print('\t1GB is {}(should be {})'.format(size_int2, size_int))
+
+    size_int3 = mb_bytes * 200  # 200MB
+    size_str = mt.convert_size(size_bytes=size_int3)
+    print('\t{} is {}'.format(size_int3, size_str))
+    size_int2 = mt.convert_size_to_bytes(size_gb=0, size_mb=200)
+    print('\t200MB is {}(should be about {})'.format(size_int2, size_int3))
+
+    size_int3 = gb_bytes * 2 + mb_bytes * 500  # 2.5 GB
+    size_str = mt.convert_size(size_bytes=size_int3)
+    print('\t{} is {}'.format(size_int3, size_str))
+    size_int2 = mt.convert_size_to_bytes(size_gb=2, size_mb=500)
+    print('\t2.5GB is {}(should be {})'.format(size_int2, size_int3))
+    return
+
+
+def size_remain_greater_than_test():
+    mt.get_function_name_and_line(ack=True, tabs=0)
+    if mt.is_windows():
+        print('checking if size remain on C: >= 1.5GB:')
+        mt.size_remain_greater_than(partition='C:', gb=1, mb=500, ack=True)
+        print('checking if size remain on C: >= 100GB and 10MB')
+        mt.size_remain_greater_than(partition='C:', gb=100, mb=10, ack=True)
+        print('checking if size remain on C: >= 500MB')
+        mt.size_remain_greater_than(partition='C:', gb=0, mb=500, ack=True)
+        print('checking if size remain on E: >= 500MB')
+        mt.size_remain_greater_than(partition='E:', gb=0, mb=500, ack=True)
+        print('checking if size remain on C: >= 2500MB(2.5GB)')
+        mt.size_remain_greater_than(partition='C:', gb=0, mb=2500, ack=True)
+    else:
+        print('checking if size remain on /dev/root >= 1.5GB:')
+        mt.size_remain_greater_than(partition='/dev/root', gb=1, mb=500, ack=True)
+    return
+
+
+def get_emoji_test():
+    mt.get_function_name_and_line(ack=True, tabs=0)
+    emojis = mt.EMOJIS.get_emojis()
+    for em in emojis:
+        print('\tthis is a {} {}'.format(em.name, mt.get_emoji(em)))
+    return
+
+
+def has_admin_test():
+    mt.get_function_name_and_line(ack=True, tabs=0)
+    mt.has_admin(ack=True)
+    return
+
+
 def test_all():
     print('{}{}:'.format('-' * 5, mt.get_base_file_and_function_name(depth=1)))
     try_say_welcome()
@@ -960,5 +1065,14 @@ def test_all():
     file_or_folder_size_test()
     classFPS_test()
     get_linkable_exception_test()
+    generate_requirements_file_test()
+    run_shell_command_test()
+    run_shell_command_and_get_out_test()
+    add_sudo_to_cmd_test()
+    size_remain_test()
+    size_remain_greater_than_test()
+    convert_size_test()
+    get_emoji_test()
+    has_admin_test()
     print('{}'.format('-' * 20))
     return
