@@ -76,7 +76,7 @@ class TflPdModel(PdBaseModel):
         resize and change dtype to predefined params
         :return:
         """
-        img_RGB = cvt.BGR_img_to_RGB(cv_img)
+        img_RGB = cvt.bgr_to_rgb(cv_img)
 
         if self.model_cfg['normalize_RGB']:
             center = 127.5
