@@ -8,7 +8,6 @@ import cv2
 
 
 def get_cv_version(ack: bool = False, tabs: int = 1) -> str:
-    """ see get_cv_version_test() """
     """ see get_cv_version_test() """
     string = mt.add_color('{}* OpenCv Version {}'.format(tabs * '\t', cv2.getVersionString()), ops=mt.SUCCESS_C)
     string += mt.add_color(' - GPU detected ? ', ops=mt.SUCCESS_C)
@@ -67,8 +66,7 @@ def display_open_cv_image(
         loc: (tuple, str) = None,
         resize: (float, int, str, tuple) = None,
         header: str = None,
-        save_path: str = None,
-        tabs: int = 1,
+        save_path: str = None
 ) -> int:
     """
     :param img: cv image in numpy array or list
@@ -80,7 +78,6 @@ def display_open_cv_image(
     :param resize: None, float>0, tuple=(w,h), str='fs'
     :param header: text to add at the top left
     :param save_path: if not none, saves image to this path
-    :param tabs:
     :return key clicked
     e.g. using key:
     if k == ord('q'):
@@ -94,7 +91,7 @@ def display_open_cv_image(
     if header is not None:
         add_header(img, header, bg_font_scale=1)
     if save_path is not None:
-        save_img(path=save_path, img=img, ack=True, tabs=tabs)
+        save_img(path=save_path, img=img, ack=True)
     cv2.imshow(title, img)
     if loc is not None:
         if mt.is_str(loc):
@@ -179,28 +176,47 @@ def move_cv_img_by_str(img: np.array, win_title: str, where: str = pyplt.Locatio
     return
 
 
-def unpack_list_imgs_to_big_image(imgs: list, grid: tuple) -> np.array:
+def unpack_list_imgs_to_big_image(imgs: list, grid: tuple, separator_c: str = 'r', empty_img_c: str = 'w',
+                                  empty_img_txt: str = 'None') -> np.array:
     """
     :param imgs: list of cv images
     :param grid: the layout you want as output
         (1,len(imgs)): 1 row
         (len(imgs),1): 1 col
         (2,2): 2x2 grid - supports len(imgs)<=4 but not more
+        (3,3): 3x3 grid - supports len(imgs)<=9 but not more
+    :param separator_c: separator color
+    :param empty_img_c: color of empty image if empty image on grid
+    :param empty_img_txt: text written on empty image if empty image on grid
     see unpack_list_imgs_to_big_image_test()
     """
     for i in range(len(imgs)):
         imgs[i] = list_to_cv_image(imgs[i])
         if len(imgs[i].shape) == 2:  # if gray - see as rgb
-            imgs[i] = gray_scale_img_to_BGR_form(imgs[i])
+            imgs[i] = gray_to_bgr(imgs[i])
 
     imgs_n = len(imgs)
     if imgs_n == 1:
         big_img = imgs[0]
     else:
-        padding_bgr = list(pyplt.get_BGR_color('red'))
+        total_slots = grid[0] * grid[1]
+        assert imgs_n <= total_slots, 'grid has {} total_slots, but len(imgs)={}'.format(total_slots, imgs_n)
+
+        padding_bgr = list(pyplt.get_bgr_color(separator_c))
         height, width, cnls = imgs[0].shape
         rows, cols = grid
-        big_img = np.zeros(shape=(height * rows, width * cols, cnls), dtype='uint8') + 255  # white big image
+        big_img = np.zeros(shape=(height * rows, width * cols, cnls), dtype='uint8')
+
+        empty_img = np.zeros(shape=(height, width, cnls), dtype='uint8')
+        empty_img[:, :] = pyplt.get_bgr_color(empty_img_c)
+
+        add_text(empty_img, header=empty_img_txt, pos=(width // 2 - len(empty_img_txt) * 13, height // 2),
+                 with_rect=False, text_color='black')
+
+        for i in range(len(imgs), total_slots):  # create an empty image for redundant grid spaces
+            imgs.append(empty_img)
+
+        # big_img[:, :] = pyplt.get_bgr_color(empty_img_c)
 
         row_ind, col_ind = 1, 1
         for i, img in enumerate(imgs):
@@ -231,7 +247,9 @@ def display_open_cv_images(
         grid: tuple = (1, 2),
         header: str = None,
         save_path: str = None,
-        tabs: int = 1,
+        separator_c: str = 'r',
+        empty_img_c: str = 'w',
+        empty_img_txt: str = 'None'
 ) -> int:
     """
     :param imgs: list of RGB or gray scale images
@@ -245,7 +263,9 @@ def display_open_cv_images(
         grid slots must be >= len(imgs)
     :param header: text to add at the top left
     :param save_path: if not none, saves image to this path
-    :param tabs:
+    :param separator_c: separator color
+    :param empty_img_c: color of empty image if empty image on grid
+    :param empty_img_txt: text written on empty image if empty image on grid
     :return key clicked
     e.g. using key:
     if k == ord('q'):
@@ -258,60 +278,118 @@ def display_open_cv_images(
     if imgs_n > 0:
         total_slots = grid[0] * grid[1]
         assert imgs_n <= total_slots, 'grid has {} total_slots, but len(imgs)={}'.format(total_slots, imgs_n)
-        big_img = unpack_list_imgs_to_big_image(imgs, grid=grid)
+        big_img = unpack_list_imgs_to_big_image(imgs, grid=grid, separator_c=separator_c, empty_img_c=empty_img_c,
+                                                empty_img_txt=empty_img_txt)
         key = display_open_cv_image(big_img, ms=ms, title=title, loc=loc, resize=resize, header=header,
-                                    save_path=save_path, tabs=tabs)
+                                    save_path=save_path)
     return key
 
 
-def gray_scale_img_to_BGR_form(gray_img: np.array) -> np.array:
+def gray_to_bgr(gray_img: np.array) -> np.array:
     """
     :param gray_img: from shape (x,y) - 1 channel (gray)
     e.g 480,640
     :return: RGB form image e.g 480,640,3. no real colors added - just shape as RGB
     see gray_to_BGR_and_back_test()
     """
-    BGR_image = cv2.cvtColor(gray_img, cv2.COLOR_GRAY2BGR)
-    return BGR_image
+    if is_gray(gray_img):
+        bgr_image = cv2.cvtColor(gray_img, cv2.COLOR_GRAY2BGR)
+    else:
+        mt.exception_error(e='image shape isn\'t gray shape: {}'.format(gray_img.shape))
+        bgr_image = None
+    return bgr_image
 
 
-def BGR_img_to_gray(bgr_img: np.array) -> np.array:
+def bgr_to_gray(bgr_img: np.array, to_bgr_form: bool = False) -> np.array:
     """
     :param bgr_img: from shape (x,y,3) - 3 channels
+    :param to_bgr_form: if needed gray in 3 channels
     e.g 480,640,3
     :return: gray image e.g 480,640. colors are replaced to gray colors
     see gray_to_BGR_and_back_test()
     """
-    gray = cv2.cvtColor(bgr_img, cv2.COLOR_BGR2GRAY)
+    if is_bgr_or_rgb(bgr_img):
+        gray = cv2.cvtColor(bgr_img, cv2.COLOR_BGR2GRAY)
+        if to_bgr_form:
+            gray = cv2.cvtColor(gray, cv2.COLOR_GRAY2BGR)
+    else:
+        mt.exception_error(e='image shape isn\'t bgr shape: {}'.format(bgr_img.shape))
+        gray = None
     return gray
 
 
-def BGR_img_to_RGB(bgr_img: np.array) -> np.array:
+def bgr_to_rgb(bgr_img: np.array) -> np.array:
     """
     :param bgr_img: from shape (x,y,3) - 3 channels
     e.g 480,640,3
     :return: rgb image
     see BGR_img_to_RGB_and_back_test()
     """
-    rgb_img = cv2.cvtColor(bgr_img, cv2.COLOR_BGR2RGB)
+    if is_bgr_or_rgb(bgr_img):
+        rgb_img = cv2.cvtColor(bgr_img, cv2.COLOR_BGR2RGB)
+    else:
+        mt.exception_error(e='image shape isn\'t rgb shape: {}'.format(bgr_img.shape))
+        rgb_img = None
     return rgb_img
 
 
-def RGB_img_to_BGR(rgb_img: np.array) -> np.array:
+def rgb_to_bgr(rgb_img: np.array) -> np.array:
     """
     :param rgb_img: from shape (x,y,3) - 3 channels
     e.g 480,640,3
     :return: bgr image
     see BGR_img_to_RGB_and_back_test()
     """
-    bgr_img = cv2.cvtColor(rgb_img, cv2.COLOR_RGB2BGR)
+    if is_bgr_or_rgb(rgb_img):
+        bgr_img = cv2.cvtColor(rgb_img, cv2.COLOR_RGB2BGR)
+    else:
+        mt.exception_error(e='image shape isn\'t rgb shape: {}'.format(rgb_img.shape))
+        bgr_img = None
     return bgr_img
 
 
 class CameraWu:
-    def __init__(self, port: int, type_cam: str = 'cv2'):
+    def __init__(self, port: (int, str), type_cam: str = 'cv2', cv2_cap: int = None, debug_shape: bool = True):
+        """
+        :param port: can be a pipe cmd command (on linux)
+        e.g. "v4l2src device=/dev/video1 ! video/x-raw, format=YUY2, width=640, height=480, framerate=60/1 !
+                videoconvert ! video/x-raw, format=BGR ! appsink drop=1"
+        :param type_cam: one of: cv2 imutils acapture
+        :param cv2_cap: cap type. valid only if type_cam == 'cv2'
+            e.g. cv2.CAP_V4L2 cv2.CAP_GSTREAMER cv2.CAP_DSHOW
+            specifies which CAP type to open
+        :param debug_shape: if True, prints on first frame read the shape
+
+        if you want debug info + change priority of videoIO:
+        linux:
+        first is debug cv and second will change the priority (first V4L2 then GSTREAMER then FFMPEG)
+        pycharm add this on configuration env variables:
+            OPENCV_VIDEOIO_DEBUG=1;OPENCV_VIDEOIO_PRIORITY_LIST=V4L2,FFMPEG,GSTREAMER
+        shell:
+            export OPENCV_VIDEOIO_DEBUG=1
+            export OPENCV_VIDEOIO_PRIORITY_LIST=V4L2,FFMPEG,GSTREAMER
+
+        or programmatically:
+        # BEFORE all imports:
+        import os
+        os.environ['OPENCV_VIDEOIO_DEBUG'] = '0'
+        # linux
+        os.environ['OPENCV_VIDEOIO_PRIORITY_LIST'] = 'V4L2,FFMPEG,GSTREAMER'
+        # windows
+        os.environ['OPENCV_VIDEOIO_PRIORITY_LIST'] = 'DSHOW,GSTREAMER,UEYE'
+
+
+        windows ffmpeg tool (best i found to replace v4l2):
+        gives you the device list:
+            ffmpeg -list_devices true -f dshow -i dummy
+        then take specific device and(e.g. my camera):
+            ffmpeg -list_options true -f dshow -i video="HD 2MP WEBCAM"
+
+
+        """
         self.port = port
         self.type_cam = type_cam
+        self.first_iter = debug_shape  # if false, no print
         try:
             pip_err = 'Error {}. pip install {}'
             failed_err = 'Failed to open CameraWu({}) on port {}'.format(type_cam, port)
@@ -345,7 +423,10 @@ class CameraWu:
                     self.cam = None
                     raise ModuleNotFoundError(pip_err.format(e, 'imutils'))
             else:  # type_cam == 'cv2'
-                self.cam = cv2.VideoCapture(port)
+                if cv2_cap is None:
+                    self.cam = cv2.VideoCapture(port)
+                else:
+                    self.cam = cv2.VideoCapture(port, cv2_cap)
                 if not self.cam.isOpened():
                     self.cam = None
                     raise Exception(failed_err)
@@ -354,9 +435,9 @@ class CameraWu:
         return
 
     @classmethod
-    def open_camera(cls, port: int, type_cam: str = 'cv2'):
+    def open_camera(cls, port: (int, str), type_cam: str = 'cv2', cv2_cap: int = None):
         try:
-            cam = cls(port, type_cam)
+            cam = cls(port=port, type_cam=type_cam, cv2_cap=cv2_cap, debug_shape=True)
             print('\tCameraWu({}) successfully open on port {}'.format(type_cam, port))
         except (ModuleNotFoundError, Exception) as e:
             mt.exception_error(e, real_exception=True, tabs=1)
@@ -413,11 +494,69 @@ class CameraWu:
                     _, frame = self.cam.read()
                     # self.cam.set(cv2.CAP_PROP_POS_FRAMES, 0)
             success = False if frame is None else True
+            if self.first_iter:
+                self.first_iter = False
+                if success:
+                    print('\tport {} frame shape is {}'.format(self.port, frame.shape))
         except Exception as e:
             mt.exception_error('CameraWu: failed capture image. e {}'.format(e), real_exception=True)
             success, frame = False, None
         return success, frame
 
+    def get_resolution(self, ack: bool = False) -> tuple:
+        """
+        :return: w_h tuple
+        """
+        if self.type_cam == 'cv2':
+            old_w = int(self.cam.get(cv2.CAP_PROP_FRAME_WIDTH))
+            old_h = int(self.cam.get(cv2.CAP_PROP_FRAME_HEIGHT))
+        elif self.type_cam == 'imutils':
+            old_w = int(self.cam.stream.get(cv2.CAP_PROP_FRAME_WIDTH))
+            old_h = int(self.cam.stream.get(cv2.CAP_PROP_FRAME_HEIGHT))
+        else:  # acapture not supported yet
+            old_w, old_h = None, None
+        if ack:
+            print('\tport {}: resolution is {}'.format(self.port, (old_w, old_h)))
+        return old_w, old_h
+
+    def set_resolution(self, w: int = 640, h: int = 480, ack: bool = True) -> bool:
+        """
+        :param w: width
+        :param h: height
+        :param ack:
+        check camera supports new res.
+        my cam worked on (w=1920, h=1080), (w=1280, h=720), (w=640, h=480)
+        on windows: go to camera on start menu, in it click settings and the resolutions supported
+            should be written there
+        :return: if resolution changed successfully
+        V4L and imutils fails to change
+        """
+        success = True
+        old_res = self.get_resolution(ack=False)
+
+        if self.type_cam == 'cv2':
+            self.cam.set(cv2.CAP_PROP_FRAME_WIDTH, w)
+            self.cam.set(cv2.CAP_PROP_FRAME_HEIGHT, h)
+
+        elif self.type_cam == 'imutils':
+            self.cam.stream.set(cv2.CAP_PROP_FRAME_WIDTH, w)
+            self.cam.stream.set(cv2.CAP_PROP_FRAME_HEIGHT, h)
+        else:  # acapture not supported yet
+            pass
+        # test if it changed successfully  #
+        new_res = self.get_resolution(ack=False)
+
+        # new_w, new_h = self.get_resolution(ack=False)
+        if old_res == new_res or new_res != (w, h):
+            mt.exception_error(e='resolution did not change. it is {}'.format(new_res))
+            mt.exception_error(e='it\'s possible you camera doesn\'t support {}'.format((w, h)))
+            mt.exception_error(e='Noticed that on gstreamer it changes but get_resolution show the default')
+            success = False
+        else:
+            if ack:
+                print('\tport {}: {} -> {}'.format(self.port, old_res, new_res, w, h))
+        return success
+
 
 def add_text(
         cv_img: np.array,
@@ -431,7 +570,7 @@ def add_text(
     """
     :param cv_img:
     :param header:
-    :param pos:
+    :param pos: tuple of x,y
     :param text_color: str
     :param with_rect:
     :param bg_color: str
@@ -440,6 +579,10 @@ def add_text(
     see add_text_test()
     """
     x, y = pos
+    if not mt.is_int(x) or not mt.is_int(y):
+        x, y = int(x), int(y)
+        pos = (x, y)
+
     if bg_font_scale == 1:  # big font and scale
         font: int = cv2.FONT_HERSHEY_DUPLEX
         font_scale: float = 1
@@ -456,11 +599,11 @@ def add_text(
         text_w, text_h = text_size
         rect_pt1 = (x, y)
         rect_pt2 = (x + text_w, y + text_h - 20)
-    text_color = pyplt.get_BGR_color(text_color)
+    text_color = pyplt.get_bgr_color(text_color)
 
     # print(pos, (x + text_w, y + text_h))
     if with_rect:
-        bg_color = pyplt.get_BGR_color(bg_color)
+        bg_color = pyplt.get_bgr_color(bg_color)
         cv2.rectangle(cv_img, pt1=rect_pt1, pt2=rect_pt2, color=bg_color, thickness=-1)
     cv2.putText(cv_img, text=header, org=pos, fontFace=font, fontScale=font_scale, color=text_color,
                 thickness=font_thickness)
@@ -511,12 +654,11 @@ def add_header(
     return
 
 
-def get_dims_from_cap(cap: cv2.VideoCapture) -> tuple:
+def get_dims_from_cap_cv2(cap: cv2.VideoCapture) -> tuple:
     """
     :param cap:
     frames size as (int, int)
     :return:
-    see Mp4_creator_test()
     """
     out_dims = None
     if cap.isOpened():
@@ -526,11 +668,22 @@ def get_dims_from_cap(cap: cv2.VideoCapture) -> tuple:
     return out_dims
 
 
-def get_frames_from_cap(cap: cv2.VideoCapture) -> int:
+def get_dims_from_cap_imutils(cap) -> tuple:
+    """
+    :param cap:
+    frames size as (int, int)
+    :return:
+    """
+    orig_width = int(cap.stream.get(cv2.CAP_PROP_FRAME_WIDTH))
+    orig_height = int(cap.stream.get(cv2.CAP_PROP_FRAME_HEIGHT))
+    out_dims = (orig_width, orig_height)
+    return out_dims
+
+
+def get_frames_from_cap_cv2(cap: cv2.VideoCapture) -> int:
     """
     :param cap:
     :return:
-    see Mp4_creator_test()
     """
     video_total_frames = None
     if cap.isOpened():
@@ -538,34 +691,157 @@ def get_frames_from_cap(cap: cv2.VideoCapture) -> int:
     return video_total_frames
 
 
-class Mp4_creator:
-    def __init__(self, out_full_path: str, out_fps: float, out_dims: tuple, tabs: int = 1):
+def get_frames_from_cap_imutils(cap) -> int:
+    """
+    :param cap:
+    :return:
+    """
+    video_total_frames = int(cap.stream.get(cv2.CAP_PROP_FRAME_COUNT)) - 1
+    return video_total_frames
+
+
+def get_fps_from_cap_cv2(cap: cv2.VideoCapture) -> float:
+    fps = None
+    if cap.isOpened():
+        fps = cap.get(cv2.CAP_PROP_FPS)
+    return fps
+
+
+def get_fps_from_cap_imutils(cap) -> float:
+    fps = cap.stream.get(cv2.CAP_PROP_FPS)
+    return fps
+
+
+def get_video_duration_from_cap_cv2(cap: cv2.VideoCapture) -> float:
+    """
+    the direct approach doesn't work:
+    duration = cap.get(cv2.CAP_PROP_POS_MSEC)
+    :param cap:
+    :return:
+    """
+    duration_seconds = None
+    if cap.isOpened():
+        frame_count = get_frames_from_cap_cv2(cap)
+        fps = get_fps_from_cap_cv2(cap)
+        duration_seconds = frame_count / fps
+        # print('{:.2f}'.format(duration_seconds))
+        duration_seconds = round(duration_seconds, 2)  # round to 2 digits precision
+    return duration_seconds
+
+
+def get_video_duration_from_cap_imutils(cap) -> float:
+    """
+    the direct approach doesn't work:
+    duration = cap.get(cv2.CAP_PROP_POS_MSEC)
+    :param cap:
+    :return:
+    """
+    frame_count = get_frames_from_cap_imutils(cap)
+    fps = get_fps_from_cap_imutils(cap)
+    duration_seconds = frame_count / fps
+    # print('{:.2f}'.format(duration_seconds))
+    duration_seconds = round(duration_seconds, 2)  # round to 2 digits precision
+    return duration_seconds
+
+
+def get_video_details(video_path: str, ack: bool = True) -> (tuple, int, float, float, str):
+    """
+    uses cv2.VideoCapture
+    :param video_path:
+    :param ack:
+    :return:
+    """
+    out_dims, video_total_frames, fps, duration, size = None, None, None, None, None
+    if os.path.exists(video_path):
+        cap = cv2.VideoCapture(video_path)
+        if cap.isOpened():
+            out_dims = get_dims_from_cap_cv2(cap)
+            video_total_frames = get_frames_from_cap_cv2(cap)
+            fps = get_fps_from_cap_cv2(cap)
+            duration = get_video_duration_from_cap_cv2(cap)
+            size = mt.file_or_folder_size(path=video_path)
+            if ack:
+                print('Video {} details:'.format(os.path.abspath(video_path)))
+                print('\t{} frames'.format(video_total_frames))
+                print('\tframe size {}'.format(out_dims))
+                print('\t{} fps'.format(fps))
+                print('\tduration {} seconds'.format(duration))
+                print('\tfile size {}'.format(size))
+            cap.release()
+        else:
+            mt.exception_error('cap is closed.', real_exception=False)
+    else:
+        mt.exception_error('file not found', real_exception=False)
+    return out_dims, video_total_frames, fps, duration, size
+
+
+class VideoCreator:
+    # more codecs http://arahna.de/opencv-save-video/
+    CODECS_TO_EXT = {
+        'mp4v': 'mp4',
+        'MJPG': 'avi',
+        'XVID': 'avi',
+        'H264': 'mkv',
+        'X264': 'mkv',
+        'PIM1': 'avi',
+        # 'FLV1': 'flv',  # a problem with this one
+        'DIVX': 'avi',  # guessed the extension
+        'MP42': 'avi',  # guessed the extension
+        'DIV3': 'avi',  # guessed the extension
+    }
+
+    def __init__(
+            self,
+            out_full_path: str,
+            out_fps: float,
+            out_dims: tuple,
+            codec: str,
+            tabs: int = 1
+    ):
         """
-        :param out_full_path:
-        :param out_fps:
-        :param out_dims:
+        :param out_full_path: path to save the video
+            if extension is not given, assigns an extension by the codec if coded exists in codec_dict
+        :param out_fps: fps of video
+            e.g. if your camera is working on 40 FPS and out_fps=20.0 and you recorded 5 seconds:
+                out video has 40*5=200 frames. each 20 will be displayed for 1 second -> video len(time)=10 seconds
+        :param out_dims: size of a frame in the video
+        :param codec: e.g. mp4v, mjpg, h264, x264, xvid ...
         :param tabs:
-        see Mp4_creator_test()
+        see video_creator_mp4_test()
+        mp4v -> file.mp4
+        MJPG or XVID-> file.avi
+        H264 or X264 -> file.mkv
+        # if u see error like:
+        Failed to load OpenH264 library: openh264-1.8.0-win64.dll
+        look for that file online, download and place next to the py file
         """
-        # fourcc = cv2.VideoWriter_fourcc(*'mp4v')
-        fourcc = cv2.VideoWriter_fourcc(c1='m', c2='p', c3='4', c4='v')
 
         self.out_full_path = out_full_path
+        file_extension = os.path.splitext(out_full_path)[-1]
+        if file_extension == '':
+            # no extension specified
+            if codec in VideoCreator.CODECS_TO_EXT:
+                self.out_full_path += '.{}'.format(VideoCreator.CODECS_TO_EXT[codec])
+
         self.out_fps = out_fps
         self.out_dims = out_dims
+        self.codec = codec
         self.tabs = tabs
         self.frames_count = 0
+
         if not os.path.exists(os.path.dirname(out_full_path)):
             mt.exception_error('Cant create cv2.VideoWriter', real_exception=False, tabs=self.tabs)
             mt.exception_error(mt.NOT_FOUND.format(os.path.dirname(out_full_path)), real_exception=False,
                                tabs=self.tabs)
             self.result = None
         else:
+            fourcc = cv2.VideoWriter_fourcc(*self.codec)
             self.result = cv2.VideoWriter(
-                filename=out_full_path,
+                filename=self.out_full_path,
                 fourcc=fourcc,
-                fps=out_fps,
-                frameSize=out_dims
+                fps=self.out_fps,
+                frameSize=self.out_dims,
+                isColor=1  # 0 not working
             )
         return
 
@@ -576,10 +852,12 @@ class Mp4_creator:
         return
 
     def __str__(self):
-        string = '{}Mp4_creator\n'.format(self.tabs * '\t')
+        string = '{}VideoCreatorObject\n'.format(self.tabs * '\t')
         string += '{}\ttarget file path: {}\n'.format(self.tabs * '\t', self.out_full_path)
         string += '{}\tfps: {}\n'.format(self.tabs * '\t', self.out_fps)
-        string += '{}\tout_dims: {}'.format(self.tabs * '\t', self.out_dims)
+        string += '{}\tout_dims: {}\n'.format(self.tabs * '\t', self.out_dims)
+        string += '{}\tcodec: {}\n'.format(self.tabs * '\t', self.codec)
+        # string += '{}\twith_color: {}\n'.format(self.tabs * '\t', self.with_color)
         return string
 
     def add_frame(self, frame: np.array, ack: bool = False, tabs: int = 1) -> None:
@@ -602,7 +880,20 @@ class Mp4_creator:
         if self.result is not None:
             self.result.release()
             self.result = None
-            print('{}File ready({} frames): {}'.format(self.tabs * '\t', self.frames_count, self.out_full_path))
+            out_dims, video_total_frames, fps, duration, size = get_video_details(self.out_full_path, ack=False)
+            if any(x is None for x in [out_dims, video_total_frames, fps, duration, size]):
+                mt.exception_error('failed to open file - something went wrong')
+            else:
+                info_msg = '{}File ready(frames={}, dims={}, size={}, duration={}seconds, fps={}): {}'.format(
+                    self.tabs * '\t',
+                    video_total_frames,
+                    out_dims,
+                    size,
+                    duration,
+                    fps,
+                    os.path.abspath(self.out_full_path)
+                )
+                print(mt.add_color(info_msg, ops=['g']))
         return
 
 
@@ -642,34 +933,38 @@ def cuda_on() -> bool:
 def get_gpu_devices_count() -> int:
     """
     if open-cv build from source with gpu support returns GPUs detected (0 if no GPU)
-    :return:
+    :return: -1 for Exception, 0 for none and >0 #devices
     """
-    count = cv2.cuda.getCudaEnabledDeviceCount()
+    try:
+        count = cv2.cuda.getCudaEnabledDeviceCount()
+    except Exception as e:
+        mt.exception_error('e {}'.format(e), real_exception=True)
+        count = -1
     return count
 
 
-def sketch_image(cv_BGR: np.array, sigma_s: float = 50, sigma_r: float = 0.07, shade_factor: float = 0.08) -> (
+def sketch_image(cv_bgr: np.array, sigma_s: float = 50, sigma_r: float = 0.07, shade_factor: float = 0.08) -> (
         np.array, np.array):
     """
-    :param cv_BGR:
+    :param cv_bgr:
     :param sigma_s: Range between 0 to 200
     :param sigma_r: Range between 0 to 1
     :param shade_factor: Range between 0 to 0.1
     :return:
     """
     sketch_gray, sketch_color = None, None
-    if len(cv_BGR.shape) == 3:
-        sketch_gray, sketch_color = cv2.pencilSketch(cv_BGR, sigma_s=sigma_s, sigma_r=sigma_r,
+    if len(cv_bgr.shape) == 3:
+        sketch_gray, sketch_color = cv2.pencilSketch(cv_bgr, sigma_s=sigma_s, sigma_r=sigma_r,
                                                      shade_factor=shade_factor)
     else:
-        mt.exception_error(e='image must be BGR: image.shape={}'.format(cv_BGR.shape))
+        mt.exception_error(e='image must be BGR: image.shape={}'.format(cv_bgr.shape))
 
     return sketch_gray, sketch_color
 
 
 def invert_image(cv_img: np.array) -> np.array:
     """
-    :param cv_img: gray\BGR - invert colors
+    :param cv_img: gray/BGR - invert colors
     :return:
     """
     invert = cv2.bitwise_not(cv_img)
@@ -713,12 +1008,15 @@ def get_color_maps(n: int = None) -> list:
     return cmaps.tolist()
 
 
-def add_color_map(cv_img: np.array, color_map: int = cv2.COLORMAP_AUTUMN) -> np.array:
+def add_color_map(cv_img: np.array, color_map: int = cv2.COLORMAP_AUTUMN, invert_base: bool = False) -> np.array:
     """
-    :param cv_img: bgr\gray -> same result
+    :param cv_img: bgr/gray -> same result
     :param color_map: one of the values from get_color_maps():
+    :param invert_base: work on the invert of the image
     :return:
     """
+    if invert_base:
+        cv_img = invert_image(cv_img)
     color_image = cv2.applyColorMap(cv_img, color_map)
     return color_image
 
@@ -736,7 +1034,7 @@ def draw_plus(cv_img: np.array, plus_center_xy: tuple, plus_color: str = 'black'
     :param text_up: up/down relative to the plus
     :return:
     """
-    plus_cBGR = pyplt.get_BGR_color(plus_color)
+    plus_cBGR = pyplt.get_bgr_color(plus_color)
 
     x, y = plus_center_xy
     xy1 = (x, y - pixels)
@@ -768,7 +1066,7 @@ def draw_x(cv_img: np.array, x_center_xy: tuple, x_color: str = 'black', pixels:
     :param text_up: up/down relative to the plus
     :return:
     """
-    plus_cBGR = pyplt.get_BGR_color(x_color)
+    plus_cBGR = pyplt.get_bgr_color(x_color)
 
     x, y = x_center_xy
     pixels = int(pixels / np.sqrt(2))
@@ -788,6 +1086,30 @@ def draw_x(cv_img: np.array, x_center_xy: tuple, x_color: str = 'black', pixels:
     return cv_img
 
 
+def set_pointer_cursor() -> None:
+    """
+    changes the cursor to and 'arrow' making it easier to select the roi points (instead of X cursor)
+    if it fails check out this fix:
+    https://stackoverflow.com/questions/58612306/how-to-fix-importerror-dll-load-failed-while-importing-win32api
+    let a: [installation directory of Anaconda]/Lib/site-packages/pywin32_system32
+    let b: C:/Windows/System32
+    basically copy 2 dlls from folder a to b
+    
+    if it fails and you dont want to fix the dlls, just send cursor_arrow=false
+    """
+    if mt.is_windows():
+        # noinspection PyBroadException
+        try:
+            # noinspection PyPackageRequirements
+            import win32api
+            import win32con
+            win32api.SetCursor(win32api.LoadCursor(0, win32con.IDC_ARROW))
+        except Exception as e:
+            # pass  # pass cus every movement of the mouse - it reaches here
+            mt.exception_error(e, real_exception=True)
+    return
+
+
 def get_roi_2dpoints(cv_img: np.array, resize: float = None, mark_color: str = 'g',
                      add_center_text: bool = False, cursor_arrow: bool = False) -> list:
     """
@@ -803,24 +1125,19 @@ def get_roi_2dpoints(cv_img: np.array, resize: float = None, mark_color: str = '
     resize_inv = None if resize is None else 1 / resize
     cv_img_clean = cv_img.copy()  # clone to delete
     if len(cv_img_clean.shape) == 2:  # gray - convert to 3 channel gray
-        cv_img_clean = gray_scale_img_to_BGR_form(cv_img_clean)
+        cv_img_clean = gray_to_bgr(cv_img_clean)
     cv_img_temp = cv_img_clean.copy()  # clone not to draw on original image
 
     # noinspection PyUnusedLocal
-    def get_roi_xy2(event, x, y, flags, param):
+    def get_roi_xy(event, x, y, flags, param):
         """
         more events at https://docs.opencv.org/3.4/d0/d90/group__highgui__window__flags.html
         [cv2.EVENT_MOUSEMOVE, cv2.EVENT_LBUTTONDOWN, cv2.EVENT_RBUTTONDOWN, cv2.EVENT_MBUTTONDOWN,
          cv2.EVENT_LBUTTONUP, cv2.EVENT_RBUTTONUP, cv2.EVENT_MBUTTONUP, cv2.EVENT_LBUTTONDBLCLK,
          cv2.EVENT_RBUTTONDBLCLK, cv2.EVENT_MBUTTONDBLCLK, cv2.EVENT_MOUSEWHEEL, cv2.EVENT_MOUSEHWHEEL]
         """
-        if cursor_arrow and event == cv2.EVENT_MOUSEMOVE and mt.is_windows():
-            try:
-                import win32api
-                import win32con
-                win32api.SetCursor(win32api.LoadCursor(0, win32con.IDC_ARROW))
-            except Exception as e:
-                mt.exception_error(e, real_exception=True)
+        if cursor_arrow and event == cv2.EVENT_MOUSEMOVE:
+            set_pointer_cursor()
 
         if event == cv2.EVENT_LBUTTONDOWN:
             if resize_inv is not None:
@@ -835,7 +1152,7 @@ def get_roi_2dpoints(cv_img: np.array, resize: float = None, mark_color: str = '
     title = "select_roi: img shape {} ('d' to delete last, 'q' to finish)".format(cv_img_temp.shape)
 
     cv2.namedWindow(title)
-    cv2.setMouseCallback(title, get_roi_xy2)
+    cv2.setMouseCallback(title, get_roi_xy)
 
     for i in range(10 ** 10):
         k = display_open_cv_image(cv_img_temp, title=title, ms=1, loc='tl', resize=resize)
@@ -874,24 +1191,19 @@ def get_named_roi_2dpoints(cv_img: np.array, names_colors: dict, resize: float =
     resize_inv = None if resize is None else 1 / resize
     cv_img_clean = cv_img.copy()  # clone to delete
     if len(cv_img_clean.shape) == 2:  # gray - convert to 3 channel gray
-        cv_img_clean = gray_scale_img_to_BGR_form(cv_img_clean)
+        cv_img_clean = gray_to_bgr(cv_img_clean)
     cv_img_temp = cv_img_clean.copy()  # clone not to draw on original image
 
     # noinspection PyUnusedLocal
-    def get_roi_xy2(event, x, y, flags, param):
+    def get_roi_xy(event, x, y, flags, param):
         """
         more events at https://docs.opencv.org/3.4/d0/d90/group__highgui__window__flags.html
         [cv2.EVENT_MOUSEMOVE, cv2.EVENT_LBUTTONDOWN, cv2.EVENT_RBUTTONDOWN, cv2.EVENT_MBUTTONDOWN,
          cv2.EVENT_LBUTTONUP, cv2.EVENT_RBUTTONUP, cv2.EVENT_MBUTTONUP, cv2.EVENT_LBUTTONDBLCLK,
          cv2.EVENT_RBUTTONDBLCLK, cv2.EVENT_MBUTTONDBLCLK, cv2.EVENT_MOUSEWHEEL, cv2.EVENT_MOUSEHWHEEL]
         """
-        if cursor_arrow and event == cv2.EVENT_MOUSEMOVE and mt.is_windows():
-            try:
-                import win32api
-                import win32con
-                win32api.SetCursor(win32api.LoadCursor(0, win32con.IDC_ARROW))
-            except Exception as e:
-                mt.exception_error(e, real_exception=True)
+        if cursor_arrow and event == cv2.EVENT_MOUSEMOVE:
+            set_pointer_cursor()
 
         if event == cv2.EVENT_LBUTTONDOWN:
             nonlocal current_idx
@@ -916,7 +1228,7 @@ def get_named_roi_2dpoints(cv_img: np.array, names_colors: dict, resize: float =
     def draw_next_info(_cv_img: np.array, _name: str, _color: str) -> None:
         h = cv_img.shape[0]
         # draw a black rectangle
-        cv2.rectangle(_cv_img, pt1=(5, h - 25), pt2=(250, h - 5), color=pyplt.get_BGR_color('black'), thickness=-1)
+        cv2.rectangle(_cv_img, pt1=(5, h - 25), pt2=(250, h - 5), color=pyplt.get_bgr_color('black'), thickness=-1)
         text_color = 'g' if _color in ['black', 'k'] else _color  # rect is black so don't allow black text color
         # add info text on the rect
         add_text(_cv_img, header='{}: {}'.format(_name, _color), pos=(5, h - 9), text_color=text_color, with_rect=False)
@@ -926,7 +1238,7 @@ def get_named_roi_2dpoints(cv_img: np.array, names_colors: dict, resize: float =
                                                                                                     names, colors)
 
     cv2.namedWindow(title)
-    cv2.setMouseCallback(title, get_roi_xy2)
+    cv2.setMouseCallback(title, get_roi_xy)
     draw_next_info(cv_img_temp, _name=names[current_idx], _color=colors[current_idx])
 
     for i in range(10 ** 10):
@@ -947,3 +1259,52 @@ def get_named_roi_2dpoints(cv_img: np.array, names_colors: dict, resize: float =
             break
     cv2.destroyAllWindows()
     return named_points
+
+
+def is_gray(cv_img: np.array) -> bool:
+    """ check if 2 channels image """
+    gray_img = (len(cv_img.shape) == 2)
+    return gray_img
+
+
+def is_bgr_or_rgb(cv_img: np.array) -> bool:
+    """ check if 3 channels image """
+    img_3_cnls = (len(cv_img.shape) == 3)
+    return img_3_cnls
+
+
+def align_images(imgs: list, pad_colors: list, new_w: int = None, new_h: int = None) -> list:
+    """
+    :param imgs: list of cv images from different sizes.
+    :param pad_colors: list of padding colors - recommended different colors if used for matching.
+    e.g.
+        if i just want to display side by side give a list with one color. e.g. pad_colors=['w']
+        if i just want to compare, it might be better different colors. e.g. pad_colors=['w', 'g', 'b']
+            len(pad_colors) can be smaller than len(imgs). cyclic colors
+    :param new_w: if None - use max_w from images, else int: will be used if bigger than all images w
+    :param new_h: if None - use max_h from images, else int: will be used if bigger than all images h
+    :return: list of cv images of the SAME size. smaller ones got padding on the w or h
+    """
+    colors_bgr = [pyplt.get_bgr_color(color_str=pad_color) for pad_color in pad_colors]
+    max_h = -1 if new_h is None else new_h
+    max_w = -1 if new_w is None else new_w
+
+    for img in imgs:
+        h, w = img.shape[:2]
+        if h > max_h:
+            max_h = h
+        if w > max_w:
+            max_w = w
+
+    imgs_same_size = []
+    for i, img in enumerate(imgs):
+        h, w = img.shape[:2]
+        img_p = cv2.copyMakeBorder(
+            img,
+            top=0, bottom=max_h - h, left=0, right=max_w - w,
+            borderType=cv2.BORDER_CONSTANT, value=colors_bgr[i % len(colors_bgr)]
+        )
+        imgs_same_size.append(img_p)
+
+    # wu.cvt.display_open_cv_images(imgs_same_size)
+    return imgs_same_size
