@@ -136,11 +136,9 @@ def display_open_cv_image_loop_test():
             img=img,
             ms=1,  # not blocking
             title=title,
-            loc=loc,  # start from x =70 y = 0
+            loc=loc if i == 0 else None,  # move just first iter
             resize=resize
         )
-        if i == 0:  # move just first iter
-            loc = None
     cv2.destroyAllWindows()
     # wu.delete_file(file=mtt.SO_LOGO_PATH, ack=True)
     return
@@ -242,7 +240,7 @@ def move_cv_img_by_str_test():
 def unpack_list_imgs_to_big_image_test():
     wu.get_function_name(ack=True, tabs=0)
     img = load_img_from_web(mtt.SO_LOGO)
-    gray = cvt.BGR_img_to_gray(img)
+    gray = cvt.bgr_to_gray(img)
     big_img = cvt.unpack_list_imgs_to_big_image(
         imgs=[img, gray, img],
         grid=(2, 2)
@@ -264,17 +262,21 @@ def unpack_list_imgs_to_big_image_test():
 def display_open_cv_images_test():
     wu.get_function_name(ack=True, tabs=0)
     img = load_img_from_web(mtt.SO_LOGO)
-    title = '2x1 grid'
+
+    title = '3x1 grid(1 empty)'
     print('\tVisual test: {}'.format(title))
     loc1 = (0, 0)
     cvt.display_open_cv_images(
         imgs=[img, img],
-        ms=1,  # blocking
+        ms=1,
         title='{} loc={}'.format(title, loc1),
         loc=loc1,
         resize=1.5,
-        grid=(2, 1),
+        grid=(3, 1),  # one empty frame
         header='{} loc={}'.format(title, loc1),
+        separator_c='aqua',
+        empty_img_c='orange',
+        empty_img_txt='No image'
     )
     # loc2 = pyplt.Location.BOTTOM_CENTER.value
     loc2 = 'bc'
@@ -303,13 +305,11 @@ def display_open_cv_images_loop_test():
             imgs=[img, img],
             ms=1,  # blocking
             title=title,
-            loc=loc,
+            loc=loc if i == 0 else None,  # move just first iter
             resize=None,
             grid=(2, 1),
             header=None
         )
-        if i == 0:  # move just first iter
-            loc = None
     cv2.destroyAllWindows()
     # wu.delete_file(file=mtt.SO_LOGO_PATH, ack=True)
     return
@@ -319,9 +319,9 @@ def gray_to_BGR_and_back_test():
     wu.get_function_name(ack=True, tabs=0)
     img = load_img_from_web(mtt.SO_LOGO)
     print(wu.to_str(img, '\timgRGB'))
-    gray = cvt.BGR_img_to_gray(img)
+    gray = cvt.bgr_to_gray(img)
     print(wu.to_str(img, '\timg_gray'))
-    img = cvt.gray_scale_img_to_BGR_form(gray)
+    img = cvt.gray_to_bgr(gray)
     print(wu.to_str(img, '\timgRGB'))
     # wu.delete_file(file=mtt.SO_LOGO_PATH, ack=True)
     return
@@ -331,9 +331,9 @@ def BGR_img_to_RGB_and_back_test():
     wu.get_function_name(ack=True, tabs=0)
     imgBGR1 = load_img_from_web(mtt.SO_LOGO)
     print(wu.to_str(imgBGR1, '\timgBGR'))
-    imgRGB = cvt.BGR_img_to_RGB(imgBGR1)
+    imgRGB = cvt.bgr_to_rgb(imgBGR1)
     print(wu.to_str(imgRGB, '\timgRGB'))
-    imgBGR2 = cvt.RGB_img_to_BGR(imgRGB)
+    imgBGR2 = cvt.rgb_to_bgr(imgRGB)
     print(wu.to_str(imgBGR2, '\timgBGR2'))
 
     cvt.display_open_cv_images(
@@ -350,6 +350,43 @@ def BGR_img_to_RGB_and_back_test():
     return
 
 
+def CameraWu_all_res_test():
+    CameraWu_res_test('cv2')
+    CameraWu_res_test('imutils')
+    return
+
+
+def CameraWu_res_test(type_cam: str):
+    wu.get_function_name(ack=True, tabs=0)
+    port = 0
+    cam = cvt.CameraWu.open_camera(port=port, type_cam=type_cam)
+    if cam is not None:
+        title = 'CameraWu_test({}) on port {}'.format(cam.type_cam, cam.port)
+        for w, h in [(1920, 1080), (1280, 720), (640, 480), (640, 360)]:
+            success_res = cam.set_resolution(w=w, h=h, ack=False)
+
+            if success_res:
+                if type_cam == 'imutils':
+                    wu.sleep(2)  # allow time to change before reading frame
+                success, cv_img = cam.read_img()
+                for i in range(5):  # sometimes first image is black
+                    success, cv_img = cam.read_img()
+                if success:
+                    k = cvt.display_open_cv_image(
+                        img=cv_img,
+                        ms=BLOCK_MS_NORMAL,
+                        title=title,
+                        loc=pyplt.Location.TOP_CENTER.value,
+                        resize=None,
+                        header='res {}x{}'.format(w, h)
+                    )
+                    if k == ord('q'):
+                        wu.exception_error('q was clicked. breaking loop')
+                        # break
+                    cv2.destroyAllWindows()
+    return
+
+
 def CameraWu_test(type_cam: str):
     WITH_SLEEP = False
     ports = [0, 1, 13]
@@ -365,18 +402,22 @@ def CameraWu_test(type_cam: str):
         for i in range(ITERS_CAM_TEST):
             fps.start()
             success, cv_img = cam.read_img()
+
             if WITH_SLEEP:
                 wu.sleep(1)
 
             if success:
-                cvt.display_open_cv_image(
+                k = cvt.display_open_cv_image(
                     img=cv_img,
                     ms=1,
                     title=title,
-                    loc=pyplt.Location.CENTER_CENTER,
+                    loc=pyplt.Location.TOP_CENTER.value,
                     resize=None,
                     header='{}/{}'.format(i + 1, ITERS_CAM_TEST)
                 )
+                if k == ord('q'):
+                    wu.exception_error('q was clicked. breaking loop')
+                    break
             fps.update()
         fps.finalize()
     cv2.destroyAllWindows()
@@ -438,7 +479,50 @@ def add_header_test():
     return
 
 
-def Mp4_creator_test():
+def get_video_details_test():
+    wu.get_function_name(ack=True, tabs=0)
+    vid_name = mtt.DOG1
+    url = mtt.VIDEOS_D[vid_name]
+    video_path = get_vid_from_web(name=vid_name)
+    if not os.path.exists(video_path):
+        wu.exception_error(wu.NOT_FOUND.format(video_path), real_exception=False)
+        return
+    print('Video {} details({}):'.format(vid_name, url))
+    cvt.get_video_details(video_path=video_path, ack=True)
+    return
+
+
+def get_video_details_imutils_test():
+    try:
+        # noinspection PyPackageRequirements,PyUnresolvedReferences
+        from imutils import video  # pip install imutils
+        wu.get_function_name(ack=True, tabs=0)
+        vid_name = mtt.DOG1
+        url = mtt.VIDEOS_D[vid_name]
+        video_path = get_vid_from_web(name=vid_name)
+        if not os.path.exists(video_path):
+            wu.exception_error(wu.NOT_FOUND.format(video_path), real_exception=False)
+            return
+        print('Video {} details({}):'.format(vid_name, url))
+        cap = video.VideoStream(src=video_path).start()
+        out_dims = cvt.get_dims_from_cap_imutils(cap)
+        video_total_frames = cvt.get_frames_from_cap_imutils(cap)
+        fps = cvt.get_fps_from_cap_imutils(cap)
+        duration = cvt.get_video_duration_from_cap_imutils(cap)
+        size = wu.file_or_folder_size(path=video_path)
+        print('Video {} details:'.format(os.path.abspath(video_path)))
+        print('\t{} frames'.format(video_total_frames))
+        print('\tframe size {}'.format(out_dims))
+        print('\t{} fps'.format(fps))
+        print('\tduration {} seconds'.format(duration))
+        print('\tfile size {}'.format(size))
+        cap.stop()
+    except ModuleNotFoundError:
+        pass
+    return
+
+
+def video_creator_mp4_test():
     wu.get_function_name(ack=True, tabs=0)
     # now open video file
     vid_name = mtt.DOG1
@@ -447,24 +531,25 @@ def Mp4_creator_test():
     if not os.path.exists(video_path):
         wu.exception_error(wu.NOT_FOUND.format(video_path), real_exception=False)
         return
+
+    out_dims, video_total_frames, fps, duration, size = cvt.get_video_details(video_path=video_path, ack=True)
+
     cap = cv2.VideoCapture(video_path)
-    if cap.isOpened():
-        out_dims = cvt.get_dims_from_cap(cap)
-        video_total_frames = cvt.get_frames_from_cap(cap)
-        print('\tvid {} has {} frames'.format(vid_name, video_total_frames))
-        print('\tvid size is {}'.format(out_dims))
-    else:
+    if not cap.isOpened():
         wu.exception_error('cap is closed.', real_exception=False)
         return
 
     out_dir = '{}/create_mp4_test'.format(mtt.VIDEOS_OUTPUTS)
     wu.create_dir(out_dir)
-    out_fp = '{}/{}_output.mp4'.format(out_dir, vid_name)
+    # out_fp = '{}/{}_output.mp4'.format(out_dir, vid_name)
+    out_fp = '{}/{}_output'.format(out_dir, vid_name)  # not specifying extension
 
-    mp4_creator = cvt.Mp4_creator(
-        out_full_path=out_fp,
+    mp4_creator = cvt.VideoCreator(
+        out_full_path=out_fp,  # since coded is mp4v, file will get .mp4 extension
         out_fps=20.0,
-        out_dims=out_dims
+        out_dims=out_dims,
+        codec='mp4v',
+        tabs=1
     )
     print(mp4_creator)
 
@@ -473,7 +558,7 @@ def Mp4_creator_test():
         if i % int(video_total_frames / 10) != 0:  # s
             # do only 10 frames
             continue
-        print('\tframe {}/{}:'.format(i + 1, video_total_frames))
+        # print('\tframe {}/{}:'.format(i + 1, video_total_frames))
         # print('\t\t{}'.format(wu.to_str(frame)))
         if success:
             cvt.add_header(
@@ -485,7 +570,7 @@ def Mp4_creator_test():
             )
             cvt.display_open_cv_image(frame, ms=1, title=vid_name, loc=None,
                                       header='{}/{}'.format(i + 1, video_total_frames))
-            mp4_creator.add_frame(frame, ack=True, tabs=2)
+            mp4_creator.add_frame(frame, ack=False, tabs=2)
 
     cap.release()
     mp4_creator.finalize()
@@ -493,6 +578,66 @@ def Mp4_creator_test():
     return
 
 
+def video_creator_size_test():
+    wu.get_function_name(ack=True, tabs=0)
+    # now open video file
+    vid_name = mtt.DOG1
+    video_path = get_vid_from_web(name=vid_name)
+
+    if not os.path.exists(video_path):
+        wu.exception_error(wu.NOT_FOUND.format(video_path), real_exception=False)
+        return
+
+    out_dims, video_total_frames, fps, duration, size = cvt.get_video_details(video_path=video_path, ack=True)
+    out_dir = '{}/video_creator_size_test'.format(mtt.VIDEOS_OUTPUTS)
+    wu.create_dir(out_dir)
+    codecs = list(cvt.VideoCreator.CODECS_TO_EXT.keys())
+    sizes = []
+    for codec in codecs:
+        cap = cv2.VideoCapture(video_path)
+        if not cap.isOpened():
+            wu.exception_error('cap is closed.', real_exception=False)
+            return
+
+        out_fp = '{}/{}_output_{}'.format(out_dir, vid_name, codec)  # not specifying extension
+        mp4_creator = cvt.VideoCreator(
+            out_full_path=out_fp,  # since coded is mp4v, file will get .mp4 extension
+            out_fps=20.0,
+            out_dims=out_dims,
+            codec=codec,
+            tabs=1
+        )
+        # print(mp4_creator)
+
+        for i in range(video_total_frames):
+            success, frame = cap.read()
+            if i % int(video_total_frames / 10) != 0:  # s
+                # do only 10 frames
+                continue
+            if success:
+                cvt.add_header(
+                    frame,
+                    header='create_mp4_test frame {}/{}'.format(i + 1, video_total_frames),
+                    loc=pyplt.Location.BOTTOM_LEFT.value,
+                    text_color=pyplt.get_random_color(),
+                    bg_color=pyplt.get_random_color(),
+                )
+                cvt.display_open_cv_image(frame, ms=1, title=vid_name, loc=None,
+                                          header='{}/{}'.format(i + 1, video_total_frames))
+                mp4_creator.add_frame(frame, ack=False, tabs=2)
+
+        cap.release()
+        mp4_creator.finalize()
+        file_size = wu.file_or_folder_size(path=mp4_creator.out_full_path)
+        sizes.append(file_size)
+        cv2.destroyAllWindows()
+
+    print('codes to file size summary:')
+    for codec, file_size in zip(codecs, sizes):
+        print('\t{}: {}'.format(codec, file_size))
+    return
+
+
 def get_aspect_ratio_test():
     wu.get_function_name(ack=True, tabs=0)
     cv_img_fake = np.zeros(shape=(480, 640, 3))
@@ -581,7 +726,7 @@ def cuda_on_gpu_test():
     * delete both zips and extracted open cv and open cv contrib
     7 after checking on pycharm, open cv with GPU support should work but no autocomplete.
     pip install mypy
-    stubgen -m cv2 -o ENV_PATH\Lib\site-packages\cv2
+    stubgen -m cv2 -o ENV_PATH/Lib/site-packages/cv2
     # if the stubgen fails, run as administrator
     # rename cv2.pyi created to __init__.pyi
     # all done
@@ -623,7 +768,7 @@ def sketch_image_test():
     wu.get_function_name(ack=True, tabs=0)
     print('\tVisual test:')
     cv_img = load_img_from_web(mtt.DOG, ack=False)
-    skg, skc = cvt.sketch_image(cv_BGR=cv_img)
+    skg, skc = cvt.sketch_image(cv_bgr=cv_img)
 
     cvt.add_header(skg, header='sketch_gray')
     cvt.add_header(skc, header='sketch_color')
@@ -632,7 +777,7 @@ def sketch_image_test():
         imgs=[cv_img, skg, skc],
         ms=BLOCK_MS_NORMAL,
         title='sketch_image_test',
-        loc='cc',
+        loc='tl',
         resize=0.7,
         grid=(1, 3),
         header=None
@@ -650,7 +795,7 @@ def invert_image_test():
     invert_bgr = cvt.invert_image(cv_img=cv_img)
     cvt.add_header(invert_bgr, header='invert_bgr')
 
-    cv_gray = cvt.BGR_img_to_gray(cv_img)
+    cv_gray = cvt.bgr_to_gray(cv_img)
     invert_gray = cvt.invert_image(cv_img=cv_gray)
     cvt.add_header(invert_gray, header='invert_gray')
 
@@ -658,7 +803,7 @@ def invert_image_test():
         imgs=[cv_img, invert_bgr, invert_gray],
         ms=BLOCK_MS_NORMAL,
         title='invert_image_test',
-        loc='cc',
+        loc='tl',
         resize=0.7,
         grid=(1, 3),
         header=None
@@ -673,7 +818,7 @@ def add_color_map_test():
     wu.get_function_name(ack=True, tabs=0)
     print('\tVisual test:')
     cv_img = load_img_from_web(mtt.DOG, ack=False)
-    cv_gray = cvt.BGR_img_to_gray(cv_img)
+    cv_gray = cvt.bgr_to_gray(cv_img)
     color_map = cv2.COLORMAP_PLASMA
 
     colored_img_on_bgr = cvt.add_color_map(cv_img=cv_img, color_map=color_map)
@@ -686,7 +831,7 @@ def add_color_map_test():
         imgs=[cv_img, colored_img_on_bgr, colored_img_on_gray],
         ms=BLOCK_MS_NORMAL,
         title='add_colors_to_gray_image_test',
-        loc='cc',
+        loc='tl',
         resize=0.7,
         grid=(1, 3),
         header=None
@@ -699,7 +844,7 @@ def all_color_maps_test():
     wu.get_function_name(ack=True, tabs=0)
     print('\tVisual test:')
     cv_img = load_img_from_web(mtt.DOG, ack=False)
-    cv_gray = cvt.BGR_img_to_gray(cv_img)
+    cv_gray = cvt.bgr_to_gray(cv_img)
     cmaps = cvt.get_color_maps()  # max cmaps 22
     all_images = [cv_img, cv_gray]
     for color_map in cmaps:
@@ -707,15 +852,23 @@ def all_color_maps_test():
         cvt.add_header(colored_img, header='idx {}'.format(color_map))
         all_images.append(colored_img)
 
+    all_images.append(cvt.invert_image(cv_img))  # just to align the rows
+    all_images.append(cvt.invert_image(cv_gray))
+    for color_map in cmaps:  # do again with base inverted
+        colored_img = cvt.add_color_map(cv_img=cv_gray, color_map=color_map, invert_base=True)
+        cvt.add_header(colored_img, header='idx {}'.format(color_map))
+        all_images.append(colored_img)
+
     cvt.display_open_cv_images(
         imgs=all_images,
         ms=BLOCK_MS_NORMAL,
         title='all_color_maps_test',
-        loc='cc',
+        loc='tl',
         resize=0.3,
-        grid=(3, 8),  # 24 total with the originals
+        grid=(6, 8),  # 2 + 22 + 2 + 22 = 48 total with the originals (twice)
         header=None
     )
+
     cv2.destroyAllWindows()
     return
 
@@ -736,7 +889,7 @@ def draw_plus_test():
         img=cv_img,
         ms=BLOCK_MS_NORMAL,
         title='draw_plus_test',
-        loc='cc',
+        loc='tl',
         resize=None,
         header=None
     )
@@ -760,7 +913,7 @@ def draw_x_test():
         img=cv_img,
         ms=BLOCK_MS_NORMAL,
         title='draw_plus_test',
-        loc='cc',
+        loc='tl',
         resize=None,
         header=None
     )
@@ -797,7 +950,7 @@ def get_named_roi_2dpoints_test():
 def get_named_roi_2dpoints_multi_image_test():
     wu.get_function_name(ack=True, tabs=0)
     cv_img = load_img_from_web(mtt.PERSON, ack=True)
-    _, sketch_color = cvt.sketch_image(cv_BGR=cv_img)
+    _, sketch_color = cvt.sketch_image(cv_bgr=cv_img)
     colored_img = cvt.add_color_map(cv_img=cv_img, color_map=cvt.get_color_maps(n=1)[0])
     colored_img2 = cvt.add_color_map(cv_img=cv_img, color_map=cvt.get_color_maps(n=1)[0])
 
@@ -851,6 +1004,34 @@ def get_named_roi_2dpoints_multi_image_test():
     return
 
 
+def is_bgr_and_3_cnl_img_test():
+    wu.get_function_name(ack=True, tabs=0)
+    cv_img = load_img_from_web(mtt.DOG, ack=True)  # dog.jpg(159.92 KB) of shape (576, 768, 3)
+    print('\tis_bgr_or_rgb? {}'.format(cvt.is_bgr_or_rgb(cv_img)))
+    print('\tis_gray? {}'.format(cvt.is_gray(cv_img)))
+    print('\tconverting to gray and rechecking...')
+    cv_img = cvt.bgr_to_gray(cv_img)
+    print('\tis_bgr_or_rgb? {}'.format(cvt.is_bgr_or_rgb(cv_img)))
+    print('\tis_gray? {}'.format(cvt.is_gray(cv_img)))
+    return
+
+
+def align_images_test():
+    wu.get_function_name(ack=True, tabs=0)
+    cv_img1 = load_img_from_web(mtt.DOG, ack=True)  # dog.jpg(159.92 KB) of shape (576, 768, 3)
+    cv_img2 = load_img_from_web(mtt.PERSON, ack=True)
+    cv_img3 = load_img_from_web(mtt.HORSES, ack=True)
+    cv_img3 = cvt.bgr_to_gray(cv_img3, to_bgr_form=True)
+    cv_img4 = load_img_from_web(mtt.FACES, ack=True)
+    aligned_images = cvt.align_images(imgs=[cv_img1, cv_img2, cv_img3, cv_img4], pad_colors=['b'])
+    cvt.display_open_cv_images(imgs=aligned_images, grid=(2, 2), resize=0.3, loc='tl', ms=BLOCK_MS_NORMAL,
+                               title='align_images_test')
+    aligned_images = cvt.align_images(imgs=[cv_img1, cv_img2, cv_img3, cv_img4], pad_colors=['g', 'w', 'purple'])
+    cvt.display_open_cv_images(imgs=aligned_images, grid=(2, 2), resize=0.3, loc='tl', ms=BLOCK_MS_NORMAL,
+                               title='align_images_test')
+    return
+
+
 def test_all():
     print('{}{}:'.format('-' * 5, wu.get_base_file_and_function_name()))
     get_cv_version_test()
@@ -871,7 +1052,11 @@ def test_all():
     CameraWu_cv2_test()
     CameraWu_acapture_test()
     CameraWu_imutils_test()
-    Mp4_creator_test()
+    CameraWu_all_res_test()
+    get_video_details_test()
+    get_video_details_imutils_test()
+    video_creator_mp4_test()
+    video_creator_size_test()
     get_aspect_ratio_test()
     cuda_on_gpu_test()
     sketch_image_test()
@@ -882,5 +1067,7 @@ def test_all():
     draw_x_test()
     # get_roi_2dpoints_test() # interactive test
     # get_named_roi_2dpoints_test()  # interactive test
+    is_bgr_and_3_cnl_img_test()
+    align_images_test()
     print('{}'.format('-' * 20))
     return
