@@ -296,7 +296,7 @@ MARKERS = [
 ]
 
 
-def get_RGB_color(color_str: str) -> tuple:
+def get_rgb_color(color_str: str) -> tuple:
     """
     :param color_str: e.g. "red", "blue" ...
         see COLORS_STRINGS for options
@@ -309,7 +309,7 @@ def get_RGB_color(color_str: str) -> tuple:
     return rgb
 
 
-def get_RGBA_color(color_str: str, opacity: float = 1.0, fp: int = 3) -> tuple:
+def get_rgba_color(color_str: str, opacity: float = 1.0, fp: int = 3) -> tuple:
     """
     :param color_str: e.g. "red", "blue" ...
         see COLORS_STRINGS for options
@@ -325,19 +325,19 @@ def get_RGBA_color(color_str: str, opacity: float = 1.0, fp: int = 3) -> tuple:
     return rgba
 
 
-def get_BGR_color(color_str: str) -> tuple:
+def get_bgr_color(color_str: str) -> tuple:
     """
     :param color_str: e.g. "red", "blue" ...
         see COLORS_STRINGS for options
     :return: bgr color
     see get_colors_formats_test()
     """
-    rgb = get_RGB_color(color_str)
-    bgr = RGB_to_BGR(rgb)
+    rgb = get_rgb_color(color_str)
+    bgr = rgb_to_bgr(rgb)
     return bgr
 
 
-def RGBA_to_RGB(rgba: tuple) -> tuple:
+def rgba_to_rgb(rgba: tuple) -> tuple:
     """
     :param rgba: RGBA format - tuple 1,4
     :return: rgb color format
@@ -349,18 +349,18 @@ def RGBA_to_RGB(rgba: tuple) -> tuple:
     return rgb
 
 
-def RGBA_to_BGR(rgba: tuple) -> tuple:
+def rgba_to_bgr(rgba: tuple) -> tuple:
     """
     :param rgba: RGBA format - tuple 1,4
     :return: bgr color format
     see RGBA_to_RGB_and_BGR_test()
     """
-    rgb = RGBA_to_RGB(rgba)
-    bgr = RGB_to_BGR(rgb)
+    rgb = rgba_to_rgb(rgba)
+    bgr = rgb_to_bgr(rgb)
     return bgr
 
 
-def BGR_to_RGB(bgr: tuple) -> tuple:
+def bgr_to_rgb(bgr: tuple) -> tuple:
     """
     :param bgr:
     :return: rgb
@@ -370,19 +370,19 @@ def BGR_to_RGB(bgr: tuple) -> tuple:
     return rgb
 
 
-def BGR_to_RGBA(bgr: tuple, opacity: float = 1.0) -> tuple:
+def bgr_to_rgba(bgr: tuple, opacity: float = 1.0) -> tuple:
     """
     :param bgr:
     :param opacity:
     :return: rgba
     see BGR_to_RGB_and_RGBA_test()
     """
-    rgb = BGR_to_RGB(bgr)
-    rgba = RGB_to_RGBA(rgb, opacity)
+    rgb = bgr_to_rgb(bgr)
+    rgba = rgb_to_rgba(rgb, opacity)
     return rgba
 
 
-def RGB_to_BGR(rgb: tuple) -> tuple:
+def rgb_to_bgr(rgb: tuple) -> tuple:
     """
     :param rgb:
     :return: bgr
@@ -392,7 +392,7 @@ def RGB_to_BGR(rgb: tuple) -> tuple:
     return bgr
 
 
-def RGB_to_RGBA(rgb: tuple, opacity: float = 1.0, fp: int = 3) -> tuple:
+def rgb_to_rgba(rgb: tuple, opacity: float = 1.0, fp: int = 3) -> tuple:
     """
     :param rgb:
     :param opacity:
@@ -435,9 +435,9 @@ def colors_plot(ack: bool = True, tabs: int = 1, max_time: float = None):
         if name in mt.CONST_COLOR_SHORTCUTS.values():
             shortcut = mt.get_key_by_value(mt.CONST_COLOR_SHORTCUTS, name)
             name_p = '{}({})'.format(name, shortcut)
-        print_d[str(i)] = '{}, RGB {}, BGR {}'.format(name_p, get_RGB_color(name), get_BGR_color(name))
+        print_d[str(i)] = '{}, RGB {}, BGR {}'.format(name_p, get_rgb_color(name), get_bgr_color(name))
         ax.text(xi_text, y, name_p, fontsize=(h * 0.6), horizontalalignment='left', verticalalignment='center')
-        ax.hlines(y + h * 0.1, xi_line, xf_line, color=get_RGBA_color(name), linewidth=(h * 0.8))
+        ax.hlines(y + h * 0.1, xi_line, xf_line, color=get_rgba_color(name), linewidth=(h * 0.8))
     if ack:
         mt.dict_as_table(print_d, title='colors', tabs=tabs)
     ax.set_xlim(0, X)
@@ -464,7 +464,7 @@ def get_ticks_list(x_low: [float, int], x_high: [float, int], p: float = 0.1) ->
     return x_ticks
 
 
-def get_random_RBGA_color_map(n: int, opacity: float = 1.0) -> np.array:
+def get_random_rgba_color_map(n: int, opacity: float = 1.0) -> np.array:
     """
     get colors list uniform distribution
     :param n: how many colors
@@ -486,11 +486,11 @@ def get_random_color(c_type: str = 'str') -> (str, tuple):
     if c_type == 'str':
         pass
     elif c_type == 'RGB':
-        color = get_RGB_color(color)
+        color = get_rgb_color(color)
     elif c_type == 'RGBA':
-        color = get_RGBA_color(color)
+        color = get_rgba_color(color)
     elif c_type == 'BGR':
-        color = get_BGR_color(color)
+        color = get_bgr_color(color)
     return color
 
 
@@ -903,9 +903,9 @@ def update_2d_scatter(sc: PathCollection, sc_dict: dict) -> None:
     """
     :param sc: scatter 2d pointer
     :param sc_dict: dict with data and c
-    data must exist - could be none or empty array\list.
+    data must exist - could be none or empty array/list.
         else nx2 array of x,y points
-    c optional - could be none or empty array\list.
+    c optional - could be none or empty array/list.
         don't exist, None or empty - colors for this scatter stay as they were
         list of len 1: one color for all points
         list of len n: color for each point
@@ -1546,7 +1546,7 @@ def plot_3d_iterative_dashboard_init(
         ax.set_facecolor(data_plot_i['face_color'])
 
     if 'background_color' in data_plot_i and data_plot_i['background_color'] is not None:
-        ax_background = get_RGBA_color(data_plot_i['background_color'])
+        ax_background = get_rgba_color(data_plot_i['background_color'])
         ax.w_xaxis.set_pane_color(ax_background)
         ax.w_yaxis.set_pane_color(ax_background)
         ax.w_zaxis.set_pane_color(ax_background)
@@ -1654,9 +1654,9 @@ def update_3d_scatter(sc: Path3DCollection, sc_dict: dict) -> None:
     """
     :param sc: scatter 3d pointer
     :param sc_dict: dict with data and c
-    data must exist - could be none or empty array\list.
-        else nx3 array\list of x,y,z points
-    c optional - could be none or empty array\list.
+    data must exist - could be none or empty array/list.
+        else nx3 array/list of x,y,z points
+    c optional - could be none or empty array/list.
         don't exist, None or empty - colors for this scatter stay as they were
         str, RGBA tuple or list of len 1: one color for all points
         list of len n: list of str or RGBA: color for each point
