@@ -15,9 +15,9 @@ PLT_MAX_TIME = 2  # close plt after x seconds - minimum 2
 def get_colors_formats_test():
     mt.get_function_name(ack=True, tabs=0)
     color_str = 'orange'
-    rgb = pyplt.get_RGB_color(color_str)
-    rgba = pyplt.get_RGBA_color(color_str)
-    bgr = pyplt.get_BGR_color(color_str)
+    rgb = pyplt.get_rgb_color(color_str)
+    rgba = pyplt.get_rgba_color(color_str)
+    bgr = pyplt.get_bgr_color(color_str)
     print('\tcolor {}: RGB={}, RGBA={}, BGR={}'.format(color_str, rgb, rgba, bgr))
     return
 
@@ -25,12 +25,12 @@ def get_colors_formats_test():
 def RGBA_to_RGB_and_BGR_test():
     mt.get_function_name(ack=True, tabs=0)
     color_str = 'orange'
-    rgb = pyplt.get_RGB_color(color_str)
-    bgr = pyplt.get_BGR_color(color_str)
+    rgb = pyplt.get_rgb_color(color_str)
+    bgr = pyplt.get_bgr_color(color_str)
 
-    rgba = pyplt.get_RGBA_color(color_str)
-    rgb2 = pyplt.RGBA_to_RGB(rgba)
-    bgr2 = pyplt.RGBA_to_BGR(rgba)
+    rgba = pyplt.get_rgba_color(color_str)
+    rgb2 = pyplt.rgba_to_rgb(rgba)
+    bgr2 = pyplt.rgba_to_bgr(rgba)
 
     print('\tRGBA {} - {}'.format(rgba, color_str))
     print('\tRGB {}=={} ? {}'.format(rgb, rgb2, rgb == rgb2))
@@ -41,12 +41,12 @@ def RGBA_to_RGB_and_BGR_test():
 def BGR_to_RGB_and_RGBA_test():
     mt.get_function_name(ack=True, tabs=0)
     color_str = 'orange'
-    rgb = pyplt.get_RGB_color(color_str)
-    rgba = pyplt.get_RGBA_color(color_str)
+    rgb = pyplt.get_rgb_color(color_str)
+    rgba = pyplt.get_rgba_color(color_str)
 
-    bgr = pyplt.get_BGR_color(color_str)
-    rgb2 = pyplt.BGR_to_RGB(bgr)
-    rgba2 = pyplt.BGR_to_RGBA(bgr)
+    bgr = pyplt.get_bgr_color(color_str)
+    rgb2 = pyplt.bgr_to_rgb(bgr)
+    rgba2 = pyplt.bgr_to_rgba(bgr)
     print('\tBGR {} - {}'.format(bgr, color_str))
     print('\tRGB {}=={} ? {}'.format(rgb, rgb2, rgb == rgb2))
     print('\tRGBA {}=={} ? {}'.format(rgba, rgba2, rgba == rgba2))
@@ -56,12 +56,12 @@ def BGR_to_RGB_and_RGBA_test():
 def RGB_to_RGBA_and_BGR_test():
     mt.get_function_name(ack=True, tabs=0)
     color_str = 'orange'
-    bgr = pyplt.get_BGR_color(color_str)
-    rgba = pyplt.get_RGBA_color(color_str)
+    bgr = pyplt.get_bgr_color(color_str)
+    rgba = pyplt.get_rgba_color(color_str)
 
-    rgb = pyplt.get_RGB_color(color_str)
-    bgr2 = pyplt.RGB_to_BGR(rgb)
-    rgba2 = pyplt.RGB_to_RGBA(rgb)
+    rgb = pyplt.get_rgb_color(color_str)
+    bgr2 = pyplt.rgb_to_bgr(rgb)
+    rgba2 = pyplt.rgb_to_rgba(rgb)
     print('\tRGB {} - {}'.format(rgb, color_str))
     print('\tRGBA {}=={} ? {}'.format(rgba, rgba2, rgba == rgba2))
     print('\tBGR {}=={} ? {}'.format(bgr, bgr2, bgr == bgr2))
@@ -85,7 +85,7 @@ def get_ticks_list_test():
 
 def get_random_RGBA_color_map_test():
     mt.get_function_name(ack=True, tabs=0)
-    random_RBGA_colors = pyplt.get_random_RBGA_color_map(n=3)
+    random_RBGA_colors = pyplt.get_random_rgba_color_map(n=3)
     print(mt.to_str(random_RBGA_colors, '\trandom_color_map'))
     return
 
@@ -191,7 +191,7 @@ def add_update_2d_scatter_test():
     new_data = np.array([[2, 2], [3, 3]])  # notice data has to be in the fig world
     # colors = [pyplt.get_RGBA_color('y')]  # list of size 1 or n of RGBA colors
     # colors = pyplt.get_random_RBGA_color_map(n=2)
-    colors = [pyplt.get_random_color('RGBA'), pyplt.get_RGBA_color('b')]  # list of size 1 or n of RGBA colors
+    colors = [pyplt.get_random_color('RGBA'), pyplt.get_rgba_color('b')]  # list of size 1 or n of RGBA colors
     pyplt.update_2d_scatter(sc=sc, sc_dict={'data': new_data, 'c': colors})
     pyplt.render_plot(fig=fig, block=False)
 
@@ -267,7 +267,7 @@ def plot_2d_one_figure_iterative_test():
                 'data': mt.np_uniform(shape=(points_per_sc, 2), lows=lows, highs=highs),
                 # 'c': None,  # color doesn't have to change
                 # 'c': [pyplt.get_RGBA_color('g')]  # color could be a list of a color
-                'c': pyplt.get_random_RBGA_color_map(points_per_sc)  # color could be a list of n color
+                'c': pyplt.get_random_rgba_color_map(points_per_sc)  # color could be a list of n color
             }
             new_data_plot_i.append(new_data_i)
 
@@ -399,7 +399,7 @@ def plot_2d_many_figures_iterative_test():
                     'data': mt.np_uniform(shape=(points_per_sc, 2), lows=lows, highs=highs),
                     # 'c': None,  # color doesn't have to change
                     # 'c': [pyplt.get_RGBA_color('g')]  # color could be a list of a color
-                    'c': pyplt.get_random_RBGA_color_map(points_per_sc)  # color could be a list of n color
+                    'c': pyplt.get_random_rgba_color_map(points_per_sc)  # color could be a list of n color
                 }
                 new_data_plot_i.append(new_data_i)
             new_data_all_plots.append(new_data_plot_i)
@@ -495,7 +495,7 @@ def plot_2d_iterative_dashboard_test():
         'yticks': None,
         'datum': [
             {
-                'c': pyplt.get_RGBA_color('r'),
+                'c': pyplt.get_rgba_color('r'),
                 'label': 'plot0_data0',
                 'marker': pyplt.get_random_marker(),
                 'marker_s': 125,
@@ -629,7 +629,7 @@ def plot_2d_iterative_dashboards_test():
                     'data': mt.np_random_integers(low=0, high=480, size=(20, 2)),
                     # 'c': None,  # color doesn't have to change
                     # 'c': [pyplt.get_RGBA_color('g')]  # color could be a list of a color
-                    'c': pyplt.get_random_RBGA_color_map(points_per_sc)  # color could be a list of n color
+                    'c': pyplt.get_random_rgba_color_map(points_per_sc)  # color could be a list of n color
                 }
                 new_data_plot_i.append(new_data_i)
             new_data_all_plots.append(new_data_plot_i)
@@ -703,9 +703,9 @@ def compare_images_sets_test():
         print('\tVisual test: 2 compare_images_sets')
         title = 'compare_images_sets_test()'
         imgBGR = cvtt.load_img_from_web(mtt.SO_LOGO)
-        imgRGB = cvt.BGR_img_to_RGB(imgBGR)
-        img_gray = cvt.BGR_img_to_gray(imgBGR)
-        img_gray = cvt.gray_scale_img_to_BGR_form(img_gray)
+        imgRGB = cvt.bgr_to_rgb(imgBGR)
+        img_gray = cvt.bgr_to_gray(imgBGR)
+        img_gray = cvt.gray_to_bgr(img_gray)
         # print(mt.to_str(imgBGR))
         # print(mt.to_str(imgRGB))
         # print(mt.to_str(img_gray))
@@ -754,9 +754,9 @@ def compare_images_multi_sets_squeezed_test():
         print('\tVisual test: 2 compare_images_multi_sets_squeezed')
         title = 'compare_images_multi_sets_squeezed_test()'
         imgBGR = cvtt.load_img_from_web(mtt.SO_LOGO)
-        imgRGB = cvt.BGR_img_to_RGB(imgBGR)
-        img_gray = cvt.BGR_img_to_gray(imgBGR)
-        img_gray = cvt.gray_scale_img_to_BGR_form(img_gray)
+        imgRGB = cvt.bgr_to_rgb(imgBGR)
+        img_gray = cvt.bgr_to_gray(imgBGR)
+        img_gray = cvt.gray_to_bgr(img_gray)
         # print(mt.to_str(imgBGR))
         # print(mt.to_str(imgRGB))
         # print(mt.to_str(img_gray))
@@ -813,7 +813,7 @@ def add_update_3d_scatter_test():
     mt.sleep(seconds=SLEEP)
     new_data = np.array([[2, 2, 2], [3, 3, 3]])  # notice data has to be in the fig world
     # colors = 'y' # equivalent pyplt.get_RGBA_color('y'), [pyplt.get_RGBA_color('y')]
-    colors = ['y', pyplt.get_RGBA_color('b')]  # list of size n of RGBA or str colors
+    colors = ['y', pyplt.get_rgba_color('b')]  # list of size n of RGBA or str colors
     pyplt.update_3d_scatter(sc=sc, sc_dict={'data': new_data, 'c': colors})
     pyplt.render_plot(fig=fig, block=False)
 
@@ -846,7 +846,7 @@ def plot_3d_iterative_dashboard_test():
         'zticks': pyplt.get_ticks_list(-20, 20, 0.5),
         'datum': [
             {
-                'c': pyplt.get_RGBA_color('r'),
+                'c': pyplt.get_rgba_color('r'),
                 'label': 'plot0_data0',
                 'marker': 'o',
                 'marker_s': 150,
@@ -912,7 +912,7 @@ def plot_3d_iterative_dashboard_test():
                 # 'c': None,  # color doesn't have to change
                 # 'c': pyplt.get_RGBA_color('r')  # color could be a RGBA color
                 # 'c': [pyplt.get_RGBA_color('g')]  # color could be a list of a RGBA color
-                'c': pyplt.get_random_RBGA_color_map(points_per_sc),  # color could be a list of n RGBA color
+                'c': pyplt.get_random_rgba_color_map(points_per_sc),  # color could be a list of n RGBA color
                 # 'c': 'r'  # color could be a single str color
                 # 'c': 'red'  #  color could be a single str color
                 # 'c': ['red'] * points_per_sc  # color could be a list of n string colors
